Package: GapAnalysis
Title: Conservation Indicators Using Spatial Information
Version: 2.1.0
Date: 2026-07-23
Authors@R: c(person("Dan", "Carver", role = c("aut","cre","cph"), comment = c(ORCID = "0000-0002-1344-6357"),
                    email= "carver.dan1@gmail.com"),
             person("Sarah", "Gore",role = c("aut","cph"), comment = c(ORCID = "0000-0003-0511-7884"),
                    email= "sarahluciagora@gmail.com"),
             person("Chrystian", "Sosa",role = c("aut","cph"), comment = c(ORCID = "0000-0002-3734-3248"),
                    email= "chrystiam.sossa@gmail.com"),
             person("Colin", "Khoury", role = c("aut","cph"), comment = c(ORCID = "0000-0001-7893-5744"),
                    email= "colin.khoury@gmail.com"),
             person("Julian", "Ramirez-Villegas",role = c("aut","cph"), comment = c(ORCID = "0000-0002-8044-583X"),
                    email = "J.R.Villegas@cgiar.org"),
             person("Valentin", "Stefan",role = c("ctb"), email = "valentin.stefan@ufz.de"),
             person("Harold", "Achicanoy",role = c("ctb","cph"), email = "h.a.achicanoy@cgiar.org"),
             person("Maria Victoria", "Diaz",role = c("ctb","cph"), email = "m.v.diaz@cgiar.org"),
             person("Steven", "Sotelo",role = c("ctb","cph"), email = "h.sotelo@cgiar.org"),
             person("Nora", "Castaneda-Alvarez",role = c("ctb","cph"), email = "nora.castaneda@croptrust.org"),
             person("Kaue", "De Sousa",role = c("ctb"), email = "k.desousa@cgiar.org"))
Description: Supports the assessment of the degree of conservation of taxa 
    in conservation systems, both in ex situ (in genebanks, 
    botanical gardens, and other repositories), and in situ 
    (in protected natural areas). Methods are described in 
    Carver et al. (2021) <doi:10.1111/ecog.05430>, building on 
    Khoury et al. (2020) <doi:10.1073/pnas.2007029117>, 
    Khoury et al. (2019) <doi:10.1016/j.ecolind.2018.11.016>, 
    Khoury et al. (2019) <doi:10.1111/DDI.13008>, 
    Castaneda-Alvarez et al. (2016) <doi:10.1038/nplants.2016.22>, 
    and Ramirez-Villegas et al. (2010) <doi:10.1371/journal.pone.0013497>.
URL: https://github.com/CIAT-DAPA/GapAnalysis
BugReports: https://github.com/CIAT-DAPA/GapAnalysis/issues
Depends: R (>= 4.3.0)
Imports: dataverse, dplyr, leaflet, terra
License: GPL-3
LazyData: true
Encoding: UTF-8
Suggests: sf, knitr, rmarkdown, devtools, remotes, tidyr, stringr,
        openxlsx, ggplot2, ggtext, cowplot, htmltools, htmlwidgets,
        kableExtra, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-23 18:42:30 UTC; dancarver
Author: Dan Carver [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1344-6357>),
  Sarah Gore [aut, cph] (ORCID: <https://orcid.org/0000-0003-0511-7884>),
  Chrystian Sosa [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-3734-3248>),
  Colin Khoury [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-7893-5744>),
  Julian Ramirez-Villegas [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8044-583X>),
  Valentin Stefan [ctb],
  Harold Achicanoy [ctb, cph],
  Maria Victoria Diaz [ctb, cph],
  Steven Sotelo [ctb, cph],
  Nora Castaneda-Alvarez [ctb, cph],
  Kaue De Sousa [ctb]
Maintainer: Dan Carver <carver.dan1@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-23 19:00:02 UTC
Built: R 4.5.2; ; 2026-07-23 21:14:00 UTC; unix

Package: GapAnalysis
Title: Conservation Indicators Using Spatial Information
Version: 2.0.2
Date: 2026-05-12
Authors@R: c(person("Dan", "Carver", role = c("aut","cre","cph"), comment = c(ORCID = "0000-0002-1344-6357"),
                    email= "carver.dan1@gmail.com"),
             person("Sarah", "Gore",role = c("aut","cph"), comment = c(ORCID = "0000-0003-0511-7884"),
                    email= "sarahluciagora@gmail.com"),
             person("Chrystian", "Sosa",role = c("aut","cph"), comment = c(ORCID = "0000-0002-3734-3248"),
                    email= "chrystiam.sossa@gmail.com"),
             person("Colin", "Khoury", role = c("aut","cph"), comment = c(ORCID = "0000-0001-7893-5744"),
                    email= "colin.khoury@gmail.com"),
             person("Julian", "Ramirez-Villegas",role = c("aut","cph"), comment = c(ORCID = "0000-0002-8044-583X"),
                    email = "J.R.Villegas@cgiar.org"),
             person("Valentin", "Stefan",role = c("ctb"), email = "valentin.stefan@ufz.de"),
             person("Harold", "Achicanoy",role = c("ctb","cph"), email = "h.a.achicanoy@cgiar.org"),
             person("Maria Victoria", "Diaz",role = c("ctb","cph"), email = "m.v.diaz@cgiar.org"),
             person("Steven", "Sotelo",role = c("ctb","cph"), email = "h.sotelo@cgiar.org"),
             person("Nora", "Castaneda-Alvarez",role = c("ctb","cph"), email = "nora.castaneda@croptrust.org"),
             person("Kaue", "De Sousa",role = c("ctb"), email = "k.desousa@cgiar.org"))
Description: Supports the assessment of the degree of conservation of taxa 
    in conservation systems, both in ex situ (in genebanks, 
    botanical gardens, and other repositories), and in situ 
    (in protected natural areas). Methods are described in 
    Carver et al. (2021) <doi:10.1111/ecog.05430>, building on 
    Khoury et al. (2020) <doi:10.1073/pnas.2007029117>, 
    Khoury et al. (2019) <doi:10.1016/j.ecolind.2018.11.016>, 
    Khoury et al. (2019) <doi:10.1111/DDI.13008>, 
    Castaneda-Alvarez et al. (2016) <doi:10.1038/nplants.2016.22>, 
    and Ramirez-Villegas et al. (2010) <doi:10.1371/journal.pone.0013497>.
URL: https://github.com/CIAT-DAPA/GapAnalysis
BugReports: https://github.com/CIAT-DAPA/GapAnalysis/issues
Depends: R (>= 4.3.0)
Imports: dataverse, dplyr, leaflet, terra
License: GPL-3
LazyData: true
Encoding: UTF-8
Suggests: sf, knitr, rmarkdown, devtools, remotes, tidyr, stringr,
        openxlsx, ggplot2, ggtext, cowplot, htmltools, htmlwidgets,
        kableExtra
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-12 15:10:52 UTC; dan
Author: Dan Carver [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1344-6357>),
  Sarah Gore [aut, cph] (ORCID: <https://orcid.org/0000-0003-0511-7884>),
  Chrystian Sosa [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-3734-3248>),
  Colin Khoury [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-7893-5744>),
  Julian Ramirez-Villegas [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8044-583X>),
  Valentin Stefan [ctb],
  Harold Achicanoy [ctb, cph],
  Maria Victoria Diaz [ctb, cph],
  Steven Sotelo [ctb, cph],
  Nora Castaneda-Alvarez [ctb, cph],
  Kaue De Sousa [ctb]
Maintainer: Dan Carver <carver.dan1@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 15:40:02 UTC
Built: R 4.6.0; ; 2026-05-12 19:54:28 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ERSex)
export(ERSin)
export(FCSc_mean)
export(FCSex)
export(FCSin)
export(GRSex)
export(GRSin)
export(SRSex)
export(SRSin)
export(checkEcoregion)
export(checkOccurrences)
export(checkProtectedAreas)
export(checksdm)
export(generateCounts)
export(generateEcoSelection)
export(generateGBuffers)
export(getDatasets)
importFrom(dataverse,get_file)
importFrom(dplyr,case_when)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,tibble)
importFrom(leaflet,addCircleMarkers)
importFrom(leaflet,addControl)
importFrom(leaflet,addLegend)
importFrom(leaflet,addPolygons)
importFrom(leaflet,addRasterImage)
importFrom(leaflet,addTiles)
importFrom(leaflet,leaflet)
importFrom(terra,aggregate)
importFrom(terra,as.data.frame)
importFrom(terra,buffer)
importFrom(terra,cellSize)
importFrom(terra,crop)
importFrom(terra,crs)
importFrom(terra,expanse)
importFrom(terra,extract)
importFrom(terra,intersect)
importFrom(terra,mask)
importFrom(terra,project)
importFrom(terra,rast)
importFrom(terra,rasterize)
importFrom(terra,res)
importFrom(terra,same.crs)
importFrom(terra,values)
importFrom(terra,vect)
importFrom(terra,writeRaster)
importFrom(terra,zonal)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
# Check for pandoc and knitr status
has_pandoc <- rmarkdown::pandoc_available()
save_outputs <- !isTRUE(getOption('knitr.in.progress')) && has_pandoc

## ----load-libraries-and-data--------------------------------------------------
# ── Load libraries ─────────────────────────────────────────────────────────────
library(GapAnalysis)
library(dplyr)
library(tidyr)
library(stringr)
library(terra)
library(leaflet)
library(openxlsx)
library(ggplot2)
library(ggtext)
library(cowplot)
library(htmltools)
library(htmlwidgets)
library(kableExtra)

# ── Set outputs folder ────────────────────────────────────────────────────────
# NOTE: This path must match the path set in the knit: field in the YAML header
# All files will be saved here. Update both locations if changing the path, e.g.:
#outputs_folder <- "C:/Users/you/Desktop/GapAnalysis/outputs"
outputs_folder <- "outputs"
if (!dir.exists(outputs_folder)) dir.create(outputs_folder)

# ── Source functions from GapAnalysis repo (alternative method) ───────────────
#files <- list.files("R", pattern = "\\.[rR]$", full.names = TRUE)
#for (f in files) source(f)

# ── Load data from GapAnalysis repo (alternative method) ─────────────────────
#load("data/CucurbitaData.rda")      # Occurrence data
#load("data/CucurbitaRasts.rda")     # Raster list
#load("data/ProtectedAreas.rda")     # Protected areas raster
#load("data/ecoregions.rda")         # Ecoregions data

# ── Load your own data ────────────────────────────────────────────────────────
# NOTE: To use your own occurrence data and SDM rasters, remove the hashes
# below and update the file paths.
#occData <- read.csv("path/to/your_occurrences.csv") # Local occurrence data
#sdms <- list(                                  # Local rasters read in as list
#   rast("path/to/species1_sdm.tif"),
#   rast("path/to/species2_sdm.tif"))
# NOTE: To download protected areas and ecoregions data locally, run getDatasets(). 
# This requires an active internet connection.
#getDatasets()          # Local download of protected areas and ecoregions data

# ── Load example Cucurbita data (GapAnalysis library) ────────────────────────
# NOTE: To use your own data, hash out the two lines below.
occData <- CucurbitaData                  
sdms    <- terra::unwrap(CucurbitaRasts)

# ── Prepare spatial layers ────────────────────────────────────────────────────
ecos     <- terra::vect(ecoregions)
proAreas <- terra::unwrap(ProtectedAreas)

# Standardize CRS across all spatial layers to prevent mismatch warnings
target_crs <- "+proj=longlat +datum=WGS84"
ecos     <- terra::project(ecos, target_crs)
proAreas <- terra::project(proAreas, target_crs)

# ── Prepare species list and SDM rasters ──────────────────────────────────────
taxa    <- unique(occData$species)
sdmList <- lapply(seq_along(taxa), function(i) {
  r <- sdms[[i]]
  if (terra::crs(r, proj = TRUE) != terra::crs(ecos, proj = TRUE)) {
    r <- terra::project(r, terra::crs(ecos))
  }
  r <- terra::subst(r, 0, NA)
  r[!(is.na(r) | r == 1)] <- NA
  r
})
names(sdmList) <- taxa

# Confirm species names
message("\nConfirm species for gap analysis:\n", paste(taxa, collapse = "\n"))

## ----run-gap-analysis---------------------------------------------------------
results_list <- list()

for (i in seq_along(taxa)) {
  taxon          <- taxa[i]
  occurrenceData <- occData[occData$species == taxon, ]
  sdm            <- sdms[[i]]

  if (terra::crs(sdm, proj = TRUE) != terra::crs(ecos, proj = TRUE)) {
    sdm <- terra::project(sdm, terra::crs(ecos))
  }
  sdm <- terra::subst(sdm, 0, NA)
  sdm[!(is.na(sdm) | sdm == 1)] <- NA

  # 1. Run SRSex first (on all records)
  srsex <- SRSex(taxon = taxon, occurrenceData = occurrenceData)

  # 2. Check inputs
  occurrences     <- checkOccurrences(csv = occurrenceData, taxon = taxon)
  if (!inherits(sdm, "SpatRaster")) stop("sdm is not a SpatRaster")
  sdm_checked     <- checksdm(sdm)
  proArea_cropped <- terra::crop(proAreas, terra::ext(sdm_checked))
  proArea_checked <- checkProtectedAreas(protectedArea = proArea_cropped,
                                         sdm           = sdm_checked)

  # Skip species if SDM raster has no values
  if (all(is.na(terra::values(sdm_checked)))) {
    message("Skipping ", taxon, ": SDM raster has no values.")
    next
  }

  # 3. Generate G buffers
  gBuffer <- generateGBuffers(
    taxon          = taxon,
    occurrenceData = occurrences$data,
    bufferDistM    = 50000
  )

  # 4. Ex-situ analysis
  grsex <- GRSex(taxon = taxon, sdm = sdm_checked, gBuffer = gBuffer)
  ersex <- ERSex(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    gBuffer        = gBuffer,
    ecoregions     = ecos,
    idColumn       = "ECO_NAME"
  )
  fcsex <- FCSex(taxon = taxon, srsex = srsex, grsex = grsex, ersex = ersex)

  # 5. In-situ analysis
  srsin <- SRSin(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    protectedAreas = proArea_checked
  )
  grsin <- GRSin(taxon = taxon, sdm = sdm_checked, protectedAreas = proArea_checked)
  ersin <- ERSin(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    protectedAreas = proArea_checked,
    ecoregions     = ecos,
    idColumn       = "ECO_NAME"
  )
  fcsin   <- FCSin(taxon = taxon, srsin = srsin, grsin = grsin, ersin = ersin)
  fcsmean <- FCSc_mean(taxon = taxon, fcsin = fcsin, fcsex = fcsex)

  # 6. Store results
  results_list[[taxon]] <- list(
    srsex           = srsex,
    occurrences     = occurrences,
    sdm_checked     = sdm_checked,
    proArea_checked = proArea_checked,
    eco_checked     = ecoregions,
    grsex           = grsex,
    ersex           = ersex,
    fcsex           = fcsex,
    srsin           = srsin,
    grsin           = grsin,
    ersin           = ersin,
    fcsin           = fcsin,
    fcsmean         = fcsmean
  )
}

## ----results-tables-----------------------------------------------------------
# ── Ex-situ metrics ───────────────────────────────────────────────────────────
exsitu_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species          = taxon,
    SRS_exsitu       = r$srsex$`SRS exsitu`,
    GRS_exsitu       = r$fcsex$`GRS exsitu`,
    ERS_exsitu       = r$fcsex$`ERS exsitu`,
    FCS_exsitu       = r$fcsex$`FCS exsitu`,
    FCS_exsitu_score = r$fcsex$`FCS exsitu score`
  )
}))

# ── In-situ metrics ───────────────────────────────────────────────────────────
insitu_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species          = taxon,
    SRS_insitu       = r$fcsin$`SRS insitu`,
    GRS_insitu       = r$fcsin$`GRS insitu`,
    ERS_insitu       = r$fcsin$`ERS insitu`,
    FCS_insitu       = r$fcsin$`FCS insitu`,
    FCS_insitu_score = r$fcsin$`FCS insitu score`
  )
}))

# ── Combined metrics ──────────────────────────────────────────────────────────
combined_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species         = taxon,
    FCSc_min        = r$fcsmean$FCSc_min,
    FCSc_max        = r$fcsmean$FCSc_max,
    FCSc_mean       = r$fcsmean$FCSc_mean,
    FCSc_min_class  = r$fcsmean$FCSc_min_class,
    FCSc_max_class  = r$fcsmean$FCSc_max_class,
    FCSc_mean_class = r$fcsmean$FCSc_mean_class
  )
}))

knitr::kable(exsitu_table,   digits = 2, caption = "<span style='text-align:left; display:block;'>Ex-situ gap analysis metrics</span>",  format = "html") %>% kableExtra::kable_styling()
knitr::kable(insitu_table,   digits = 2, caption = "<span style='text-align:left; display:block;'>In-situ gap analysis metrics</span>",   format = "html") %>% kableExtra::kable_styling()
knitr::kable(combined_table, digits = 2, caption = "<span style='text-align:left; display:block;'>Combined gap analysis metrics</span>",  format = "html") %>% kableExtra::kable_styling()

## ----gap-analysis-figure, include=FALSE, eval=exists("save_outputs") && save_outputs----
# plot_data <- do.call(rbind, lapply(names(results_list), function(taxon) {
#   r <- results_list[[taxon]]
#   data.frame(
#     ID        = taxon,
#     SRSex     = as.numeric(r$srsex[["SRS exsitu"]]),
#     GRSex     = as.numeric(r$grsex$results[["GRS exsitu"]]),
#     ERSex     = as.numeric(r$ersex$results[["ERS exsitu"]]),
#     FCSex     = as.numeric(r$fcsex[["FCS exsitu"]]),
#     SRSin     = as.numeric(r$srsin$results[["SRS insitu"]]),
#     GRSin     = as.numeric(r$grsin$results[["GRS insitu"]]),
#     ERSin     = as.numeric(r$ersin$results[["ERS insitu"]]),
#     FCSin     = as.numeric(r$fcsin[["FCS insitu"]]),
#     FCSc_mean = as.numeric(r$fcsmean[["FCSc_mean"]])
#   )
# }))
# 
# header_font_size     <- 7
# legend_label_font_mm <- 5
# point_stroke         <- 1.5
# other_size           <- 4.0
# fcs_size             <- 6.0
# fcsc_size            <- 10.5
# 
# all_species <- sort(unique(plot_data$ID))
# format_id <- function(id) {
#   id |>
#     str_replace_all(" var\\. ", "</i> var. <i>") |>
#     str_replace_all(" x ",      "</i> x <i>") |>
#     (\(x) paste0("<i>", x, "</i>"))()
# }
# formatted_species_levels <- format_id(all_species)
# 
# plot_data <- plot_data |>
#   mutate(
#     ID_italic  = format_id(.data$ID),
#     ID_ordered = factor(.data$ID_italic, levels = rev(formatted_species_levels))
#   )
# 
# long_df <- plot_data |>
#   pivot_longer(
#     cols      = c(SRSex, GRSex, ERSex, FCSex, SRSin, GRSin, ERSin, FCSin,
#                   FCSc_mean),
#     names_to  = "metric_raw",
#     values_to = "x"
#   ) |>
#   filter(!is.na(.data$x)) |>
#   mutate(
#     metric_base = case_when(
#       str_detect(.data$metric_raw, "FCSc") ~ "FCSc_mean",
#       TRUE ~ str_remove(.data$metric_raw, "(ex|in)$")
#     ),
#     origin_type = case_when(
#       str_detect(.data$metric_raw, "in$")  ~ "in situ",
#       str_detect(.data$metric_raw, "ex$")  ~ "ex situ",
#       TRUE                                 ~ "combined"
#     ),
#     point_size_dynamic = case_when(
#       .data$metric_base == "FCSc_mean" ~ fcsc_size,
#       .data$metric_base == "FCS"       ~ fcs_size,
#       TRUE                             ~ other_size
#     ),
#     plot_order = factor(.data$metric_base,
#                         levels = c("FCSc_mean", "FCS", "SRS", "GRS", "ERS"))
#   ) |>
#   arrange(.data$plot_order)
# 
# color_map    <- c("FCSc_mean" = "#FF0000", "FCS" = "#000000",
#                   "SRS"       = "#0000FF", "GRS" = "#800080", "ERS" = "#228B22")
# shape_map    <- c("combined" = 18, "ex situ" = 16, "in situ" = 17)
# panel_colors <- c("#ffb4b3", "#ffd380", "#ffff80", "#a8d2a8")
# 
# build_panel <- function(data, id_levels) {
#   num_levels <- length(id_levels)
#   header_y   <- num_levels + 1.25
#   ymax       <- num_levels + 2.0
#   ylim_top   <- num_levels + 2.2
# 
#   header_data <- tibble(
#     x     = c(12.5, 37.5, 62.5, 87.5),
#     label = c("<b>Urgent<br>Priority</b>", "<b>High<br>Priority</b>",
#               "<b>Medium<br>Priority</b>", "<b>Low<br>Priority</b>")
#   )
# 
#   ggplot(data, aes(x = .data$x, y = .data$ID_ordered)) +
#     annotate("rect",
#              xmin  = c(0, 25, 50, 75), xmax = c(25, 50, 75, 100),
#              ymin  = 0.5, ymax = ymax,
#              fill  = panel_colors, alpha = 0.9) +
#     geom_segment(data = data.frame(y = seq_len(num_levels)),
#                  aes(x = 0, xend = 100, y = y, yend = y),
#                  color = "grey75", linewidth = 0.5, inherit.aes = FALSE) +
#     geom_segment(data = data.frame(x = c(0, 25, 50, 75, 100)),
#                  aes(x = x, xend = x, y = 0.5, yend = ymax),
#                  color = "black", linewidth = 1.5, inherit.aes = FALSE) +
#     geom_hline(yintercept = c(0.5, num_levels + 0.5, num_levels + 2.0),
#                color = "black", linewidth = 1.5) +
#     geom_jitter(aes(color = .data$metric_base, shape = .data$origin_type,
#                     size  = .data$point_size_dynamic),
#                 stroke = point_stroke, width = 0.7, height = 0) +
#     geom_richtext(data = header_data,
#                   aes(x = .data$x, y = header_y, label = .data$label),
#                   size = header_font_size, color = "black",
#                   fill = NA, label.color = NA,
#                   lineheight = 0.9, hjust = 0.5, inherit.aes = FALSE) +
#     scale_color_manual(values = color_map, guide = "none") +
#     scale_shape_manual(values = shape_map, guide = "none") +
#     scale_size_identity() +
#     scale_x_continuous(breaks = c(0, 25, 50, 75, 100), expand = c(0, 0)) +
#     scale_y_discrete(drop = FALSE, limits = rev(id_levels)) +
#     coord_cartesian(xlim = c(0, 100), ylim = c(1, ylim_top), clip = "off") +
#     theme_minimal_grid() +
#     theme(
#       panel.grid   = element_blank(),
#       axis.title   = element_blank(),
#       axis.ticks.y = element_blank(),
#       axis.ticks.x = element_blank(),
#       axis.text.x  = element_text(size = 16),
#       axis.text.y  = element_markdown(size = 18, hjust = 1,
#                                       margin = margin(r = 15)),
#       plot.margin  = margin(t = 30, r = 20, b = 20, l = 40)
#     )
# }
# 
# main_plot <- build_panel(long_df, formatted_species_levels)
# 
# legend_data <- tribble(
#   ~label,                           ~x,    ~y,   ~shape,    ~color,     ~symbol_size,         ~label_size,          ~vjust,
#   "<b>Conservation<br>Status</b>",  0.5,   1.8,  NA_real_,  "black",    NA_real_,             legend_label_font_mm, 1,
#   "FCSc-mean",                      3.0,   1.7,  18,        "#FF0000",  fcsc_size,            legend_label_font_mm, 0.5,
#   "FCS ex situ",                    4.8,   1.7,  16,        "black",    fcs_size,             legend_label_font_mm, 0.5,
#   "SRS ex situ",                    6.6,   1.7,  16,        "#0000FF",  other_size,           legend_label_font_mm, 0.5,
#   "GRS ex situ",                    8.4,   1.7,  16,        "#800080",  other_size,           legend_label_font_mm, 0.5,
#   "ERS ex situ",                    10.2,  1.7,  16,        "#228B22",  other_size,           legend_label_font_mm, 0.5,
#   "FCS in situ",                    4.8,   1.3,  17,        "black",    fcs_size,             legend_label_font_mm, 0.5,
#   "SRS in situ",                    6.6,   1.3,  17,        "#0000FF",  other_size,           legend_label_font_mm, 0.5,
#   "GRS in situ",                    8.4,   1.3,  17,        "#800080",  other_size,           legend_label_font_mm, 0.5,
#   "ERS in situ",                    10.2,  1.3,  17,        "#228B22",  other_size,           legend_label_font_mm, 0.5
# )
# 
# legend_plot <- ggplot(legend_data, aes(x = .data$x, y = .data$y)) +
#   geom_point(aes(shape = .data$shape, color = .data$color,
#                  size  = .data$symbol_size), stroke = point_stroke) +
#   geom_richtext(aes(label = .data$label, x = .data$x + 0.15,
#                     size  = .data$label_size, vjust = .data$vjust),
#                 hjust = 0, fill = NA, label.color = NA) +
#   scale_color_identity() +
#   scale_shape_identity() +
#   scale_size_identity() +
#   theme_void() +
#   coord_cartesian(xlim = c(0, 11.5), ylim = c(0.5, 2.2))
# 
# n_taxa      <- length(formatted_species_levels)
# plot_height <- max(10, n_taxa * 0.5)
# final_plot  <- plot_grid(main_plot, legend_plot,
#                          ncol = 1, rel_heights = c(1, 0.1))
# 
# save_plot(
#   file.path(outputs_folder, "gapAnalysis_conservationPriority_figure.png"),
#   final_plot,
#   base_width  = 12,
#   base_height = plot_height,
#   dpi         = 300,
#   bg          = "white")
# 
# message("Saved: ",
#         file.path(outputs_folder, "gapAnalysis_conservationPriority_figure.png"))

## ----save-metrics-excel, include=FALSE, eval=exists("save_outputs") && save_outputs----
# # ── Build table (1 row per species, all metrics as columns) ───────
# all_metrics <- do.call(rbind, lapply(names(results_list), function(taxon) {
#   r <- results_list[[taxon]]
# 
#   clean <- function(x) {
#     df <- as.data.frame(t(unlist(x)))
#     df <- df[, !grepl("(?i)^(taxon|species)$", names(df)), drop = FALSE]
#     return(df)
#   }
# 
#   row <- cbind(
#     data.frame(Taxon = taxon),
#     clean(r$srsex),
#     clean(r$grsex$results),
#     clean(r$ersex$results),
#     clean(r$fcsex),
#     clean(r$srsin$results),
#     clean(r$grsin$results),
#     clean(r$ersin$results),
#     clean(r$fcsin),
#     clean(r$fcsmean)
#   )
#   row <- row[, !duplicated(names(row)), drop = FALSE]
#   return(row)
# }))
# 
# # ── Round numeric columns ─────────────────────────────────────────────────────
# for (col in names(all_metrics)) {
#   num <- suppressWarnings(as.numeric(as.character(all_metrics[[col]])))
#   if (all(is.na(num) == is.na(all_metrics[[col]]))) {
#     all_metrics[[col]] <- round(num, 2)
#   }
# }
# 
# # ── Clean column names ────────────────────────────────────────────────────────
# names(all_metrics) <- gsub("\\.", " ", names(all_metrics))
# 
# insert_pos  <- which(names(all_metrics) == "Area in protected areas km2")
# all_metrics <- cbind(
#   all_metrics[, seq_len(insert_pos), drop = FALSE],
#   "Area of model km2" = all_metrics[["Area of model km2"]],
#   all_metrics[, (insert_pos + 1):ncol(all_metrics), drop = FALSE]
# )
# 
# names(all_metrics) <- dplyr::recode(names(all_metrics),
#   "Total records in SDM"            = "Total records in model",
#   "FCSc_min"                        = "FCS minimum",
#   "FCSc_max"                        = "FCS maximum",
#   "FCSc_mean"                       = "FCSc mean",
#   "FCSc_min_class"                  = "FCS minimum priority category",
#   "FCSc_max_class"                  = "FCS maximum priority category",
#   "FCSc_mean_class"                 = "FCS mean priority category",
#   "Ecoregions with protected areas" = "Ecoregions within protected areas"
# )
# 
# # ── Reorder columns ───────────────────────────────────────────────────────────
# col_order <- c(
#   "Taxon", "Total records", "Total with coordinates",
#   "Total G records", "G records with coordinates",
#   "Total H records", "H records with coordinates",
#   "SRS exsitu", "G buffer areas in model km2", "Area of model km2",
#   "GRS exsitu", "Ecoregions within G buffer", "Ecoregions with records",
#   "ERS exsitu", "FCS exsitu", "FCS exsitu score",
#   "Total Observations", "Records in Protected areas", "Total records in model",
#   "SRS insitu", "Area in protected areas km2", "Area of model km2",
#   "GRS insitu", "Ecoregions within protected areas", "Ecoregions within model",
#   "ERS insitu", "FCS insitu", "FCS insitu score",
#   "FCS minimum", "FCS maximum", "FCSc mean",
#   "FCS minimum priority category", "FCS maximum priority category",
#   "FCS mean priority category"
# )
# 
# names(all_metrics) <- make.unique(names(all_metrics), sep = "__TEMP__")
# col_order_unique   <- make.unique(col_order,           sep = "__TEMP__")
# all_metrics        <- all_metrics[, col_order_unique, drop = FALSE]
# names(all_metrics) <- gsub("__TEMP__\\d+$", "", names(all_metrics))
# 
# # ── Write to Excel ────────────────────────────────────────────────────────────
# wb <- createWorkbook()
# addWorksheet(wb, "gapAnalysis_results")
# writeData(wb, "gapAnalysis_results", all_metrics, startRow = 1, startCol = 1)
# addStyle(wb, "gapAnalysis_results",
#   style      = createStyle(wrapText = TRUE, textDecoration = "bold"),
#   rows       = 1,
#   cols       = seq_len(ncol(all_metrics)),
#   gridExpand = TRUE
# )
# setRowHeights(wb, "gapAnalysis_results", rows = 1, heights = 60)
# max_width <- max(nchar(as.character(all_metrics$Taxon))) + 2
# setColWidths(wb, "gapAnalysis_results", cols = 1, widths = max_width)
# saveWorkbook(wb,
#   file      = file.path(outputs_folder, "gapAnalysis_results.xlsx"),
#   overwrite = TRUE
# )
# message("Saved: ", file.path(outputs_folder, "gapAnalysis_results.xlsx"))

## ----map-helper, include=FALSE------------------------------------------------
clean_map <- function(map, occurrenceData = NULL, taxon = NULL) {

  legend_call <- Filter(
    function(c) !is.null(c$method) && c$method == "addLegend",
    map$x$calls
  )
  labels    <- if (length(legend_call) > 0) legend_call[[1]]$args[[1]][["labels"]] else NULL
  is_insitu <- !is.null(labels) && any(grepl("Protected", labels))

  if (is_insitu) {
    layer_groups <- c("All ecos", "Eco gaps", "Distribution", "Protected areas")
  } else {
    layer_groups <- c("All ecos", "Eco gaps", "Distribution", "G buffer")
  }

  poly_idx <- 0
  map$x$calls <- lapply(map$x$calls, function(call) {
    if (!is.null(call$method) && call$method == "addPolygons") {
      poly_idx <<- poly_idx + 1
      if (poly_idx <= 2) call$args[[3]] <- layer_groups[poly_idx]
    }
    call
  })

  raster_idx <- 0
  map$x$calls <- lapply(map$x$calls, function(call) {
    if (!is.null(call$method) && call$method == "addRasterImage") {
      raster_idx <<- raster_idx + 1
      call$args[[4]] <- layer_groups[2 + raster_idx]
    }
    call
  })

  map$x$calls <- Filter(function(call) {
    !(!is.null(call$method) && call$method == "addCircleMarkers")
  }, map$x$calls)

  map$x$calls <- Filter(function(call) {
    !(!is.null(call$method) && call$method == "addLegend")
  }, map$x$calls)

  map$x$calls <- Filter(function(call) {
    if (!is.null(call$method) && call$method == "addControl") {
      html <- call$args[[1]]
      if (is.character(html) && grepl("Ecoregions", html, ignore.case = TRUE)) {
        return(FALSE)
      }
    }
    TRUE
  }, map$x$calls)

  if (!is.null(occurrenceData) && !is.null(taxon)) {
    occ   <- occurrenceData[occurrenceData$species == taxon, ]
    h_pts <- occ[occ$type == "H", ]
    g_pts <- occ[occ$type == "G", ]

    if (nrow(h_pts) > 0) {
      map <- map |>
        leaflet::addCircleMarkers(
          lng     = h_pts$longitude,
          lat     = h_pts$latitude,
          radius  = 2,
          color   = "#1184d4",
          opacity = 1,
          group   = "H points (herbarium)"
        )
    }
    if (nrow(g_pts) > 0) {
      map <- map |>
        leaflet::addCircleMarkers(
          lng     = g_pts$longitude,
          lat     = g_pts$latitude,
          radius  = 2,
          color   = "#6300f0",
          opacity = 1,
          group   = "G points (genebank)"
        )
    }
  }

  all_groups <- c(layer_groups, "H points (herbarium)", "G points (genebank)")

  map <- map |>
    leaflet::addLayersControl(
      overlayGroups = all_groups,
      options       = layersControlOptions(collapsed = FALSE)
    )

  map <- htmlwidgets::onRender(map, "
    function(el, x) {
      setTimeout(function() {
        el.querySelectorAll('.leaflet-control-layers-overlays').forEach(function(s) {
          var prev = s.previousElementSibling;
          if (prev) prev.remove();
        });
        el.querySelectorAll('.leaflet-bottom.leaflet-left .leaflet-control')
          .forEach(function(c) { c.remove(); });
      }, 300);
    }
  ")

  map
}

## ----render-maps, include=FALSE-----------------------------------------------
for (taxon in names(results_list)) {
  cat("###", taxon, "\n\n")

  print(htmltools::tagList(
    clean_map(
      map            = results_list[[taxon]]$ersex$map,
      occurrenceData = occData,
      taxon          = taxon
    )
  ))
  cat("\n\n***\n\n")

  print(htmltools::tagList(
    clean_map(
      map            = results_list[[taxon]]$ersin$map,
      occurrenceData = occData,
      taxon          = taxon
    )
  ))
  cat("\n\n***\n\n")
}

## ----save-maps-as-html, include=FALSE, eval=exists("save_outputs") && save_outputs----
# outputs_abs <- normalizePath(outputs_folder)
# tmp_lib     <- file.path(outputs_abs, "lib_tmp")
# 
# for (taxon in names(results_list)) {
#   clean_name <- tolower(gsub(" ", "_", taxon))
# 
#   # Ex-situ map
#   ersex_map <- clean_map(
#     map            = results_list[[taxon]]$ersex$map,
#     occurrenceData = occData,
#     taxon          = taxon
#   )
#   if (!is.null(ersex_map)) {
#     file_name <- paste0(clean_name, "_exSitu_map.html")
#     htmlwidgets::saveWidget(
#       widget        = ersex_map,
#       file          = file.path(outputs_abs, file_name),
#       selfcontained = TRUE,
#       libdir        = tmp_lib
#     )
#     if (dir.exists(tmp_lib)) unlink(tmp_lib, recursive = TRUE)
#     message("Saved: ", file.path(outputs_folder, file_name))
#   } else {
#     message("No ex-situ map available for: ", taxon)
#   }
# 
#   # In-situ map
#   ersin_map <- clean_map(
#     map            = results_list[[taxon]]$ersin$map,
#     occurrenceData = occData,
#     taxon          = taxon
#   )
#   if (!is.null(ersin_map)) {
#     file_name <- paste0(clean_name, "_inSitu_map.html")
#     htmlwidgets::saveWidget(
#       widget        = ersin_map,
#       file          = file.path(outputs_abs, file_name),
#       selfcontained = TRUE,
#       libdir        = tmp_lib
#     )
#     if (dir.exists(tmp_lib)) unlink(tmp_lib, recursive = TRUE)
#     message("Saved: ", file.path(outputs_folder, file_name))
#   } else {
#     message("No in-situ map available for: ", taxon)
#   }
# }

## ----complete-workflow, include=FALSE-----------------------------------------
message(
  "\n",
  "──────────────────────────────────────────────────────────────────────\n",
  "                WORKFLOW COMPLETE. WELL DONE!\n",
  "──────────────────────────────────────────────────────────────────────\n",
  "\n",
  " >>> TO SAVE WORKFLOW AS HTML: Click the [Knit] button in RStudio <<<\n",
  "     at the top of the script.\n",
  "\n",
  " The HTML will save to the folder specified in the\n",
  " knit: field of the YAML header.\n",
  "\n",
  " To change the save location, update the path in both:\n",
  "   - the knit: field in the YAML header\n",
  "   - the outputs_folder variable in Data Preparation\n",
  "\n",
  " Review your outputs folder for all saved files.\n",
  "───────────────────────────────────────────────────────────────────────"
)

#' @title Sampling representativeness score in situ
#' @name SRSin
#' @description The SRSin process calculates the proportion of all occurrences of a taxon falling within
#' the distribution model that also fall within a protected area
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#'
#'
#' @param sdm a terra rast object that
#'
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#'
#' @param protectedAreas A terra rast object the contian spatial location of protected areas.
#'
#' @return A list object containing
#' 1. results : a data frames of values summarizing the results of the function
#' 2. points : a terra vect object showing all the points present within protected areas
#' 3. map : a leaflet object showing the spatial results of the function
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' occurrenceData <- CucurbitaData
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#' #Running SRSin
#' srs_insitu <- SRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra vect extract
#' @importFrom dplyr filter tibble
#' @importFrom leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers addControl
#' @export

SRSin <- function(taxon, sdm, occurrenceData,  protectedAreas){
  # remove all points not inside of the sdm
  # then test those for presence in protected areas
  # return the proportion and the visualization of points


  # filter the occurrence data to the species of interest
  d1 <- occurrenceData |>
    dplyr::filter(species == taxon) |>
    terra::vect(geom=c("longitude", "latitude"), crs="+proj=longlat +datum=WGS84")
  # extract values from the sdm
  d1$inSDM <- terra::extract(sdm, d1, ID = FALSE)
  # points in sdm
  p1 <- d1[d1$inSDM == 1, ]
  # extract vals from protected layers
  p1$inPro <- terra::extract(protectedAreas, p1, ID=FALSE)
  # points in pro
  protectedPoints <- p1[p1$inPro== 1, ]

  # srsin
  srsin <- nrow(protectedPoints)/nrow(p1) *100

  # dataframe for export
  out_df <- dplyr::tibble(Taxon = taxon,
                          "Total Observations" = nrow(d1),
                          "Total records in SDM" = nrow(p1),
                          "Records in Protected areas" = nrow(protectedPoints),
                          "SRS insitu" = srsin)
  # quick map
  p1$color <- ifelse(is.na(p1$inPro), "#444444", "#746fae")

  map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Points within SDM inside of protected areas</h3>"
  map <- leaflet::leaflet() |>
    leaflet::addTiles() |>
    leaflet::addRasterImage(
      x = sdm,
      colors = "#47ae24"
    )

  if (nrow(p1) > 0) {
    map <- map |>
      leaflet::addCircleMarkers(
        data = p1,
        color = ~color,
        radius = 1,
        opacity = 1
      )
  }

  map <- map |>
    leaflet::addLegend(
      position = "topright",
      title = "SRS in situ",
      colors = c("#47ae24","#746fae", "#444444"),
      labels = c("Distribution","Protected Occurrences", "Non Protected Occurrences"),
      opacity = 1
    )|>
    leaflet::addControl(html = map_title, position = "bottomleft")



  # define output
  output <- list(
    results = out_df,
    points = p1,
    map = map
    )
  return(output)
}

#' @title Download datasets from the harvard dataverse repo
#' @name getDatasets
#' @description
#' Ecoregions and protected area data base are stored on a harvard dataverse repository. This functions check to see if
#' those datasets have been download and will download them if not present.
#'
#' @return A message confirming the datasets were downloaded, along with saving the files to the package's data directory.
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dataverse get_file
#' @importFrom terra rast writeRaster
#' @export

getDatasets <- function(){
  #LOADING FOLDER PARAMETERS
  out_dir <- tools::R_user_dir("GapAnalysis", which = "data")
  if(!file.exists(out_dir)){dir.create(out_dir, recursive = TRUE)}
  
  prot_dir <- paste0(out_dir, "/protectedArea")
  if(!file.exists(prot_dir)){dir.create(prot_dir, recursive = TRUE)}
  
  ecoRegion_dir <- paste0(out_dir, "/ecoRegion")
  if(!file.exists(ecoRegion_dir)){dir.create(ecoRegion_dir, recursive = TRUE)}


  # WDPA file  --------------------------------------------------------------
  proAreaPath <- paste0(prot_dir,"/wdpa_rasterize_all.tif")
  if(!file.exists(proAreaPath)){
    # raw protected areas data
    raw_tif_data <- dataverse::get_file(
      file ="wdpa_rasterized_all.tif",
      dataset = "doi:10.7910/DVN/KQVOSW",
      server = "https://dataverse.harvard.edu"
    )

    # Write the raw data to a temporary file on your disk
    temp_file_path <- tempfile(fileext = ".tif")
    writeBin(raw_tif_data, temp_file_path)

    # Step 5: Read the temporary TIFF file into R using terra
    # The 'rast()' function from the terra package reads the raster data.
    raster_data <- terra::rast(temp_file_path)
    terra::writeRaster(x = raster_data, filename = proAreaPath)
    message("Protected areas files have been downloaded from the dataverse","\n")

  } else {
    message("Protected areas file are already downloaded","\n")
  }


  # ecoregion file ----------------------------------------------------------
  ecoRegionPath <- paste0(prot_dir,"/tnc_terr_ecoregions.gpkg")
  if(!file.exists(ecoRegionPath)){
    # raw data
    raw_eco_data <- dataverse::get_file(
      file ="tnc_terr_ecoregions.gpkg",
      dataset = "doi:10.7910/DVN/WTLNRG",
      server = "https://dataverse.harvard.edu"
    )
    # Write the raw data to a temporary file on your disk
    temp_file_path <- tempfile(fileext = ".gpkg")
    writeBin(raw_eco_data, temp_file_path)

    # Step 5: Read the temporary TIFF file into R using terra
    # The 'rast()' function from the terra package reads the raster data.
    vect_data <- terra::vect(temp_file_path)
    terra::writeVector(x = vect_data, filename = ecoRegionPath)
    message("Ecoregions files have been downloaded from the dataverse","\n")

  } else {
    message("Ecoregions files are already downloaded","\n")
  }
  return(message("DATASETS WERE DOWNLOADED!"))
}

#' @title Generate buffer of G type occurrences
#' @name generateGBuffers
#' @description
#' Produces a terra vect object representing the area around the G type occurrences
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#' @param bufferDistM Distance in meters. Used to set the size of the buffered objects.
#'
#' @return A list object containing
#' 1. data : a terra vect object showing all the buffered areas around the G type occurrences
#' 2. map : a leaflet object showing the spatial results of the function
#'

#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' occurrenceData <- CucurbitaData
#'
#' #Running generateGBuffers
#' gBuffer <- generateGBuffers(taxon = taxon,
#'                     occurrenceData = occurrenceData,
#'                     bufferDistM = 50000
#'                     )
#'
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr filter
#' @importFrom terra vect crs buffer
#' @importFrom leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers
#' @export

generateGBuffers <- function(taxon, occurrenceData, bufferDistM) {
  # filter the occurrence data to the species of interest
  d1 <- occurrenceData |>
    dplyr::filter(species == taxon & type == "G") |>
    terra::vect(geom = c("longitude", "latitude"))
  terra::crs(d1) <- "epsg:4326"
  # test to see if g points are present
  if (nrow(d1) > 0) {
    # Buffer object
    d2 <- d1 |>
      terra::buffer(width = bufferDistM)
    # leaflet map of results
    map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Buffered G Occurrences</h3>"
    map <- leaflet::leaflet(d2) |>
      leaflet::addTiles() |>
      leaflet::addPolygons(
        color = "#444444",
        weight = 1,
        opacity = 1.0,
        fillOpacity = 0.5,
        fillColor = "#6300f0"
      ) |>
      leaflet::addCircleMarkers(
        data = d1,
        color = "#000",
        radius = 2,
        opacity = 1
      ) |>
      leaflet::addControl(html = map_title, position = "bottomleft")
  } else {
    #
    d2 <- "No G points present"
    # generate a empty map
    map <- leaflet::leaflet(d2)
  }

  return(list(
    data = d2,
    map = map
  ))
}

#' @title Generate initial counts of the occurrence data
#' @name generateCounts
#' @description
#' Performs data cleaning to generate a summary of all input occurrence data. These values area
#' used in the SRSex function.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#'
#' @return countsData : a data frames of values summarizing the results of the function
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' occurrenceData <- CucurbitaData
#'
#' #Running generateCounts
#' counts <- generateCounts(taxon = taxon,
#'                     occurrenceData = occurrenceData
#'                     )
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr filter select mutate group_by summarize n
#' @export

generateCounts <- function(taxon, occurrenceData){

  # define presence of usable lat long values
  dataThin <- occurrenceData |>
    dplyr::filter(species == taxon) |>
    dplyr::select(c("species", "latitude", "longitude", "type")) |>
    dplyr::mutate(hasLat = !is.na(latitude) & latitude != "\\N" & latitude != "" & !is.null(latitude) & latitude != "NULL") |>
    dplyr::mutate(hasLong = !is.na(longitude) & longitude != "\\N"& longitude != "" & !is.null(longitude)& longitude != "NULL") |>
    dplyr::mutate(hasLatLong = hasLat & hasLong)

  # set column names for counts df
  colNames <- c("species","totalRecords",	"hasLat", "hasLong","totalUseful", 	"totalGRecords",
                "totalGUseful","totalHRecords",	"totalHUseful")
  # summarize data
  tbl <- dataThin |>
    dplyr::group_by(type, hasLatLong )|>
    dplyr::summarize(total = dplyr::n())

  # generate counts df
  countsData <- data.frame(matrix(NA, nrow = 1, ncol = 9))
  colnames(countsData) <- colNames
  #assign values to counts df
  countsData$species <- taxon
  countsData$totalRecords <- nrow(dataThin)
  countsData$totalUseful <- sum((subset(tbl, hasLatLong == TRUE))$total)
  countsData$totalGRecords <- sum((subset(tbl, type == "G"))$total)
  countsData$totalGUseful <- sum((subset(tbl, type == "G" & hasLatLong == TRUE))$total)
  countsData$totalHRecords <- sum((subset(tbl, type == "H"))$total)
  countsData$totalHUseful <- sum((subset(tbl, type == "H" & hasLatLong == TRUE))$total)
  countsData$hasLat <- sum(dataThin$hasLat)
  countsData$hasLong <- sum(dataThin$hasLong)
  return(countsData)
}

#' @title Geographical representativeness score in situ
#' @name GRSin
#' @description The GRSin process provides a geographic measurement of the proportion of a species’ range that can be considered
#' to be conserved in protected areas. The GRSin compares the area of the distribution model located within protected areas versus
#' the total area of the model, considering comprehensive conservation to have been accomplished only when the entire distribution
#' occurs within protected areas.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param sdm a terra rast object that represented the expected distribution of the species
#' @param protectedAreas A terra rast object the contian spatial location of protected areas.
#'
#' @return A list object containing
#' 1. results : a data frames of values summarizing the results of the function
#' 2. protectAreaMask : a terra rast object showing all the protected areas within the distribution
#' 3. map : a leaflet object showing the spatial results of the function
#'
#' @examples
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#'
#' #Running GRSin
#' grs_insitu <- GRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra crop expanse
#' @importFrom dplyr tibble
#' @importFrom leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers
#' @export

GRSin <- function(taxon, sdm, protectedAreas){
  # total area of the SDM inside protected areas over
  # total aras of the SDM

  # crop protected areas to sdm
  pro <- terra::crop(protectedAreas, sdm)
  # mask to model
  proMask <- pro * sdm

  # generate areas
  sdmArea <- terra::expanse(sdm,unit = "km")[,2]
  proArea <- terra::expanse(proMask,unit = "km")[,2]

  # calculate the total area
  if(proArea == 0){
    grs <- 0
  }else{
    grs <- min(c(100, proArea/sdmArea*100))
  }
  # return objects
  df_output <- dplyr::tibble(Taxon = taxon,
                   'Area of model km2' = round(sdmArea, digits = 0),
                   'Area in protected areas km2' = round(proArea, digits = 0),
                   "GRS insitu" = grs)

  map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Protect areas within the SDM</h3>"
  map <- leaflet::leaflet() |>
    leaflet::addTiles() |>
    leaflet::addRasterImage(
      x = sdm,
      colors = "#47ae24"
    )|>
    leaflet::addRasterImage(
      x = proMask,
      colors = "#746fae"
    )|>
    leaflet::addLegend(
      position = "topright",
      title = "GRS in situ",
      colors = c("#47ae24","#746fae"),
      labels = c("Distribution","Protected Areas"),
      opacity = 1
    )|>
    leaflet::addControl(html = map_title, position = "bottomleft")

  # create output data
  output <- list(
    results = df_output ,
    protectAreaMask = proMask,
    map = map
  )
  return(output)
}

#' @title Geographical representativeness score ex situ
#' @name GRSex
#' @description The GRSex process provides a geographic measurement of the proportion of a species’ range
#'  that can be considered to be conserved in ex situ repositories. The GRSex uses buffers (default 50 km radius)
#'  created around each G coordinate point to estimate geographic areas already well collected within the distribution
#'  models of each taxon, and then calculates the proportion of the distribution model covered by these buffers.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param sdm a terra rast object that represented the expected distribution of the species
#' @param gBuffer A terra vect which encompases a specific buffer distance around all G points
#'
#' @return A list object containing
#' 1. results : a data frames of values summarizing the results of the function
#' 2. gGaps : a terra vect object showing buffered area about g points
#' 3. map : a leaflet object showing the spatial results of the function
#'
#'

#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' occurrenceData <- CucurbitaData
#'
#' # generate the g buffer object
#' gBuffer <- generateGBuffers(taxon = taxon,
#'                             occurrenceData = occurrenceData,
#'                             bufferDistM = 50000)
#'
#' #Running GRSex
#' grs_exsitu <- GRSex(taxon = taxon,
#'                     sdm = sdm,
#'                     gBuffer = gBuffer
#'                     )
#'
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra cellSize rasterize mask
#' @importFrom leaflet leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers
#' @importFrom dplyr tibble
#' @export

GRSex <- function(taxon, sdm, gBuffer) {
  ## all the areas of the cells
  r1 <- terra::cellSize(sdm, unit = "km")
  ## mutliple by origin. values of 1 will retain area measures
  r2 <- r1 * sdm
  totalArea <- sum(values(r2), na.rm = TRUE)

  # clause to see if any g points exist
  if (is.character(gBuffer$data)) {
    grs <- 0
    gArea <- 0
    gMap <- sdm
    map <- leaflet::leaflet()
  } else {
    ## rasterize the object
    b1 <- terra::rasterize(x = gBuffer$data, y = sdm)
    c2 <- r1 * b1 * sdm
    gArea <- sum(values(c2), na.rm = TRUE)

    # gap map
    ## reclass from NA to 0
    b2 <- b1 * -1
    gMap <- terra::mask(x = sdm, b1, inverse = TRUE)

    # clause to determine if any of the buffered area falls within predicted area
    if (gArea == 0) {
      grs <- 0
      gArea <- 0
    } else {
      #calculate GRS
      grs <- min(c(100, gArea / totalArea * 100))
    }
    # leaflet map of results
    map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>SDM areas outside of the G Buffer zone</h3>"
    map <- leaflet::leaflet() |>
      leaflet::addTiles() |>
      leaflet::addRasterImage(gMap, colors = "#47b322") |>
      leaflet::addLegend(
        position = "topright",
        title = "GRS ex situ",
        colors = c("#47ae24", "#746fae"),
        labels = c("Distribution", "Buffer G Occurrences"),
        opacity = 1
      ) |>
      leaflet::addControl(html = map_title, position = "bottomleft")
    if (grs > 0) {
      map <- map |>
        leaflet::addRasterImage(
          x = b1,
          colors = "#746fae"
        )
    }
  }

  # create data.frame with output
  out_df <- dplyr::tibble(
    Taxon = taxon,
    'Area of model km2' = totalArea,
    'G buffer areas in model km2' = gArea,
    "GRS exsitu" = grs
  )
  # # leaflet map of results
  # map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>SDM areas outside of the G Buffer zone</h3>"
  # map <- leaflet::leaflet() |>
  #   leaflet::addTiles() |>
  #   leaflet::addRasterImage(gMap, colors = "#47b322") |>
  #   leaflet::addLegend(
  #     position = "topright",
  #     title = "GRS ex situ",
  #     colors = c("#47ae24", "#746fae"),
  #     labels = c("Distribution", "Buffer G Occurrences"),
  #     opacity = 1
  #   ) |>
  #   leaflet::addControl(html = map_title, position = "bottomleft")
  # if (grs > 0) {
  #   map <- map |>
  #     leaflet::addRasterImage(
  #       x = b1,
  #       colors = "#746fae"
  #     )
  # }

  #
  output <- list(
    results = out_df,
    gGaps = gMap,
    map = map
  )
  return(output)
}

#' @title Sampling representativeness score ex situ
#' @name SRSex
#' @description The SRSex process provides a general indication of the completeness of ex situ conservation collections,
#'  calculating the ratio of germplasm accessions (G) available in ex situ repositories to reference (H) records for each taxon,
#'  making use of all compiled records, regardless of whether they include coordinates, with an ideal (i.e., comprehensive) conservation
#'  ratio of 1:1. In this and in the subsequent measurements, if no G or H records exist, taxa are automatically considered
#'  to be of high priority for further conservation action and assigned a value of 0. If there are more G than H records,
#'  SRSex is set to 100.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#'
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#'
#' @return out_df :  a data frames of values summarizing the results of the function
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#'
#' #Running SRSex
#' srs_exsitu <- SRSex(taxon = taxon,
#'                     occurrenceData = CucurbitaData
#'                     )
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr tibble n
#' @export

SRSex <- function(taxon, occurrenceData) {
  # generate the counts data for species
  sp_counts <- generateCounts(taxon = taxon,
               occurrenceData = occurrenceData)
  # clause for no h points
  if(sp_counts$totalGRecords >= 1 & sp_counts$totalHRecords == 0){
    srs <-100
  }

  #clause for no data
  if (sp_counts$totalGRecords == 0 & sp_counts$totalHRecords ==0) {
    srs <- 0
  } else {
    # clause for species with data
    srs <- min(c(100,sp_counts$totalGRecords/sp_counts$totalHRecords*100))
  }


  #create data.frame with output
  out_df <- dplyr::tibble(Taxon=sp_counts$species,
                       "Total records"=sp_counts$totalRecords,
                       "Total with coordinates"=sp_counts$totalUseful,                       "Total G records"= sp_counts$totalGRecords,
                       "G records with coordinates"=sp_counts$totalGUseful,
                       "Total H records"=sp_counts$totalHRecords,
                       "H records with coordinates"=sp_counts$totalHUseful,
                       "SRS exsitu" =srs)
  return(out_df)
}

#' @title Ecological representativeness score in situ
#' @name ERSin
#' @description The ERSin process provides an ecological measurement of the proportion of a species range
#'  that can be considered to be conserved in protected areas. The ERSin calculates the proportion of ecoregions
#'  encompassed within the range of the taxon located inside protected areas to the ecoregions encompassed
#'  within the total area of the distribution model, considering comprehensive conservation to have been accomplished
#'  only when every ecoregion potentially inhabited by a species is included within the distribution of the species
#'  located within a protected area.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param sdm a terra rast object that represented the expected distribution of the species
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#' @param protectedAreas A terra rast object the contian spatial location of protected areas.
#' @param ecoregions A terra vect object the contains spatial information on all ecoregions of interests
#' @param idColumn A character vector that notes what column within the ecoregions object should be used as a unique ID
#'
#' @return A list object containing
#' 1. results : a data frames of values summarizing the results of the function
#' 2. missingEcos : a terra vect object showing all the ecoregions within the distribution with no protected areas present
#' 3. map : a leaflet object showing the spatial results of the function
#'
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#' ecoregions <- terra::vect(ecoregions)
#'
#' #Running ERSin
#' ers_insitu <- ERSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = CucurbitaData,
#'                     protectedAreas = protectedAreas,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr tibble pull
#' @importFrom terra crop aggregate zonal
#' @importFrom leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers
#' @export

ERSin <- function(taxon, sdm, occurrenceData, protectedAreas, ecoregions, idColumn) {
  # crop protected areas to sdm
  pro <- terra::crop(protectedAreas, sdm)
  # mask to model
  proMask <- pro * sdm
  # set id column for easier indexing
  ecoregions$id_column <- as.data.frame(ecoregions)[[idColumn]]
  # aggregates spatial features
  ecoregions <- terra::aggregate(x = ecoregions, by = "id_column")

  # crop ecos to sdm
  eco <- terra::crop(ecoregions, sdm)

  # Get ecoregions in sdm
  eco$totEco <- terra::zonal(x = sdm , z = eco, fun = "sum",na.rm=TRUE) |> dplyr::pull()
  selectedEcos <- eco[eco$totEco > 0 , ]
  nEcoModel <- nrow(selectedEcos)
  # Get ecoregions in pro areas
  eco$totPro <- terra::zonal(x = proMask, z = eco, fun = "sum",na.rm=TRUE) |> dplyr::pull()
  protectedEcos <- eco[eco$totPro > 0 , ]
  nProModel <- nrow(protectedEcos)
  # get missing ecos
  missingEcos <- selectedEcos[!selectedEcos$id_column %in% protectedEcos$id_column, ]


  # calculate the ers
  if(nProModel == 0){
    ers <- 0
  }else{
    ers <- (nProModel/nEcoModel)*100
  }
  #results
  df <- dplyr::tibble(Taxon = taxon,
                   "Ecoregions within model" = nEcoModel,
                   "Ecoregions with protected areas" = nProModel,
                   "ERS insitu" = ers)
  # generate the base map
  map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Ecoregions within the SDM without Protected Area</h3>"
  map <- leaflet::leaflet() |>
    leaflet::addTiles()

  if (nrow(selectedEcos) > 0) {
    map <- map |>
      leaflet::addPolygons(data = selectedEcos,
                  color = "#444444",
                  weight = 1,
                  opacity = 1.0,
                  popup = ~id_column,
                  fillOpacity = 0.5,
                  fillColor = "#44444420")
  }

  if (nrow(missingEcos) > 0) {
    map <- map |>
      leaflet::addPolygons(data = missingEcos,
                  color = "#444444",
                  weight = 1,
                  opacity = 1.0,
                  popup = ~id_column,
                  fillOpacity = 0.5,
                  fillColor = "#f0a01f")
  }

  map <- map |>
    leaflet::addRasterImage(
      x = sdm,
      colors = "#47ae24"
    ) |>
    leaflet::addRasterImage(
      x = proMask,
      colors = "#746fae"
    )|>
    leaflet::addLegend(
      position = "topright",
      title = "ERS in situ",
      colors = c("#47ae24","#746fae", "#f0a01f", "#44444440"),
      labels = c("Distribution","Protected Areas", "Eco gaps", "All Ecos"),
      opacity = 1
    )|>
    leaflet::addControl(html = map_title, position = "bottomleft")





  # output
  output = list(
    results = df,
    missingEcos = missingEcos,
    map = map
  )
  return(output)
}

#'
#' @title Quality check on occurrences data
#' @name checkOccurrences
#' @description
#' Checks the column names, column data types, valid lat lon, and can optionally remove any duplicated lat lon records per species.
#' The cleaned and formated dataframe is returned as well as a map object show a quick reference of the points in space.
#'
#' @param csv A dataframe holding the occurrence data
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param removeDuplicated : Binary parameter. TRUE == duplication values are remove. Set to FALSE as default
#'
#' @return A list object containing
#' 1. data : a data frames of values of occurrence data in the required format
#' 2. map : a leaflet object showing the spatial results of the function
#'
#'
#' @examples
#' # example code
#'
#' ##Obtaining occurrences from example
# data(CucurbitaData)
#
# # convert the dataset for function
# taxon <- "Cucurbita_cordata"
# #Running checkOccurrences
# occurrences <- checkOccurrences(csv = CucurbitaData,
#                     taxon = taxon,
#                     removeDuplicated = FALSE
#                     )
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr mutate select
#' @importFrom terra vect
#' @importFrom leaflet leaflet addTiles addCircleMarkers addControl
#' @export

checkOccurrences <- function(csv, taxon, removeDuplicated = FALSE){

  # check column names
  ## if don't match error out
  ## provide a message of the require
  requiredCols <- c("species","latitude","longitude","type")
  # select columns of interest
  missing_columns <- setdiff(requiredCols, names(csv))
  if(length(missing_columns) == 0){
    csv <- csv |>
      dplyr::mutate(index = dplyr::row_number())|>
      dplyr::select(index, dplyr::all_of(requiredCols))
  }else{
    stop(paste("The following columns are missing ",
               missing_columns,
               ". Please add in order to move forward. Theres are case sensitive"))
  }

  # subset to the species of interest
  totalRows <- nrow(csv)
  df <- csv[csv$species == taxon, ]
  message(paste("A total of ", nrow(df),
                " out of the ", totalRows, "contained records for ", taxon))


  # change the data type of the columns
  colTypes <- lapply(df, class)
  # check and change columns as needed
  if(colTypes$species != "character"){
    df$species <- as.character(df$species)
    message(paste("Changed the data type from ", colTypes$species,
                  " to the required character" ))
  }
  if(colTypes$latitude != "numeric"){
    df$latitude <- as.numeric(df$latitude)
    message(paste("Changed the data type from ", colTypes$latitude,
                  " to the required numeric" ))
  }
  if(colTypes$longitude != "numeric"){
    df$longitude <- as.numeric(df$longitude)
    message(paste("Changed the data type from ", colTypes$longitude,
                  " to the required numeric" ))
  }
  if(colTypes$type != "character"){
    df$type <- as.character(df$type)
    message(paste("Changed the data type from ", colTypes$type,
                  " to the required character" ))
  }

  # check the coordinate range (between -90:90 and -180:180)
  valid_lat <- df$latitude >= -90 & df$latitude <= 90 & !is.na(df$latitude)
  valid_lon <- df$longitude >= -180 & df$longitude <= 180 & !is.na(df$longitude)
  # combine to a single feature
  combined_vec <- ifelse(valid_lat == FALSE | valid_lon == FALSE, FALSE, TRUE)
  # index the records
  size <- nrow(df)
  # Remove rows with missing coordinates
  df <- df[combined_vec, ]
  invalidCoords <- size - nrow(df)
  # message
  message(paste("Removed  ", invalidCoords,
                " records because lat values were outside the range of -90 to 90 or lonitude values were outside the range of -180-180",
                "or there is a no value present for latitude or longitude for a record."))


  # Check for duplicated records (all rows)
  if(removeDuplicated == TRUE){
    dups <- !duplicated(df[, requiredCols])
    nrows <- length(dups[isFALSE(dups)])
    df <- df[dups, ]
    removedDups <- size - nrow(df)
    # message
    message(paste("Removed  ", removedDups,
                  " records because of exact match of all records in the species, latitude, longitude, and type columns"))
  }

  # generate a plot of the points for a quality check
  points <- terra::vect(df, geom = c("longitude", "latitude"), crs =  "EPSG:4326")
  # terra::plot(points,  main = "Quality Points for Gap Analysis",
  #             xlab = "Longitude", ylab = "Latitude")


  map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Quality Points for Gap Analysis</h3>"
  df$color <- ifelse(df$type == "G", yes = "#6300f0", no = "#1184d4")
  # Create the Leaflet map
  map <- leaflet::leaflet(data = df) |>
    leaflet::addTiles() |>  # Adds default OpenStreetMap map tiles
    leaflet::addCircleMarkers(
      lng = ~longitude,
      lat = ~latitude,
      popup = ~paste("Point type:", ~type), # Optional: add popups
      radius = 5,          # Adjust marker size
      stroke = FALSE,
      color = ~color,
      fillOpacity = 0.8
    ) |>
    leaflet::addControl(html = map_title, position = "bottomleft") #

  message(paste("All checks completed"))

  # return
  return(list(data = df,
         map = map))
}

#' @title Cucurbita occurrences dataset
#' @name CucurbitaData
#' @docType data
#' @description This dataset is a subset of the original dataset for:
#'  C. cordata, C. digitata and C. palmata used in Khoury et al. (2019)
#' @format A data frame with 1203 rows and 4 variables:
#' \describe{
#'   \item{species}{character: Species name}
#'   \item{latitude}{numeric:Latitude in decimal format}
#'   \item{longitude}{numeric: Longitude in decimal format}
#'   \item{type}{character: Source of the record,germplasm (G) or herbarium (H)}
#' }
#' @references
#' Khoury et al. (2019) Plants, People, Planet 2(3):269-283. \doi{10.1002/ppp3.10085}
#' @source \doi{10.7910/DVN/B8YOQL}
"CucurbitaData"

#' @title Cucurbita species distribution models dataset
#' @name CucurbitaRasters
#' @docType data
#' @format terra rast object stored as a PackedSpatRaster
#' @description This dataset is a subset of species distribution models for:
#'  C. cordata, C. digitata and C. palmata used in Khoury et al., 2020
#' @references
#' Khoury et al. (2019) Diversity and Distributions 26(2):209-225. \doi{10.1111/DDI.13008}
#' @source \doi{10.7910/DVN/B8YOQL}
"CucurbitaRasts"


#' @title Ecoregions vector
#' @name ecoregions
#' @docType data
#' @format SpatVector
#' @description This dataset is a subset of the Terrestrial Ecoregions of the World
#'  shapefile was made available by the world wildlife foundation
#' @source \doi{10.7910/DVN/B8YOQL}
"ecoregions"


#' @title Protected areas dataset in raster format
#' @name  ProtectedAreas
#' @docType data
#' @format terra rast object
#' @description This dataset is a raster version of the world protected areas dataset
#'  used in Khoury et al., (2019)
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' @source \url{https://www.protectedplanet.net/en}
"ProtectedAreas"

#' @title Final conservation score ex situ
#' @name FCSex
#' @description This function calculates the average of the three ex situ conservation metrics
#'   returning a final conservation score summary table. It also assigns conservation priority categories
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param srsex A dataframe contain the results from the srsex function
#' @param grsex A dataframe contain the results from the grsex function
#' @param ersex A dataframe contain the results from the ersex function
#'
#' @return out_df : a data frames of values summarizing the results of the function
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' occurrenceData <- CucurbitaData
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#' ecoregions <- terra::unwrap(ecoregions)
#' # generate exsitu conservation summaries
#' srs_exsitu <- SRSex(taxon = taxon,
#'                     occurrenceData = CucurbitaData
#'                     )
#'
#' gBuffer <- generateGBuffers(taxon = taxon,
#'                     occurrenceData = occurrenceData,
#'                     bufferDistM = 50000
#'                     )#'
#'
#' grs_exsitu <- GRSex(taxon = taxon,
#'                     sdm = sdm,
#'                     gBuffer = gBuffer
#'                     )
#'
#' ers_exsitu <- ERSex(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     gBuffer = gBuffer,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#' #Running fcsex
#' fcs_exsitu <- FCSex(taxon = taxon,
#'                     srsex = srs_exsitu,
#'                     grsex = grs_exsitu,
#'                     ersex = ers_exsitu)
#'
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @export
#'
FCSex <- function(taxon, srsex, grsex, ersex){

  # calculate the mean across the three measures
  srs <- srsex$`SRS exsitu`
  grs <- grsex$results$`GRS exsitu`
  ers <- ersex$results$`ERS exsitu`

  # generate the mean exsitu score
  sp_fcs <- mean(c(srs,
                   ers,
                   grs), na.rm=TRUE)

  out_df <- dplyr::tibble(Taxon = taxon,
                         "SRS exsitu"= srs,
                         "GRS exsitu" = grs,
                         "ERS exsitu"= ers,
                         "FCS exsitu"=sp_fcs,
                         "FCS exsitu score" = NA)

  #assign classes (min)
  if (sp_fcs < 25) {
    score <- "UP"
  } else if (sp_fcs >= 25 & sp_fcs < 50) {
    score <- "HP"
  } else if (sp_fcs >= 50 & sp_fcs < 75) {
    score <- "MP"
  } else {
    score <- "LP"
  }
  out_df$"FCS exsitu score" <- score
  return(out_df)

}

#' @title Quality check of ecoregion dataset
#' @name checkEcoregion
#' @description
#' Checks the class, crs, if the idColumn is a unique ID,
#' @param ecoregion A terra vect object the contains spatial information on all ecoregions of interests
#' @param sdm a terra rast object that represented the expected distribution of the species
#' @param idColumn A character vector that notes what column within the ecoregions object should be used as a unique ID
#'
#' @return ecoregions : A terra vect object the contains spatial information on all ecoregions of interests
#'
#' @examples
#' ##Obtaining Raster_list
# data(CucurbitaRasts)
# ## ecoregion features
# data(ecoregions)
#
# # convert the dataset for function
# sdm <- terra::unwrap(CucurbitaRasts)$cordata
# ecoregions <- terra::vect(ecoregions)
# #Running checkEcoregion
# ecoregions <- checkEcoregion(ecoregion = ecoregions,
#                     sdm = sdm,
#                     idColumn = "ECO_NAME"
#                     )
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra vect crs project same.crs
#' @export

checkEcoregion <- function(ecoregion, sdm, idColumn){
  # check class and convert if needed
  c1 <- class(ecoregion)
  if(c1[1] !="SpatVector"){
    ecoregion <- terra::vect(ecoregion)
    message(paste("Changed the object type from ", c1,
                  " to the required object terra vector"  ))
  }

  crs <- terra::crs(ecoregion)
  # crs
  if(!terra::same.crs(crs, terra::crs("epsg:4326"))){
    ecoregion<- terra::project(x = ecoregion, y = "epsg:4326" )
    message(paste("Changed the crs from ", crs,
                  " to the required epsg:4326"  ))
  }

  # test the unique ID column
  rows <- nrow(ecoregion)
  unID <- length(ecoregion[[idColumn]][,1])
  if(rows != unID){
    message(paste("The total rows of your ecoregion object is", rows,
                  " the unique values fo the ", idColumn, " column is ", unID,
                  "these should match. Please try a new column or you may get some unexpected results"))
  }

  message(paste("All checks completed"))

  return(ecoregion)
}

#' @title Quality check on protected areas dataset
#' @name checkProtectedAreas
#' @description
#' Checks the class, values, crs, and cell size of the protected areas raster to ensure
#' these elements match those required by gap analysis functions.
#'
#' @param protectedAreas A terra rast object the contian spatial location of protected areas.
#' @param sdm a terra rast object that represented the expected distribution of the species
#'
#' @return protectedAreas : a terra rast object representing protected areas
#'
#' @examples
#'
#' ##Obtaining Raster_list
# data(CucurbitaRasts)
# ##Obtaining protected areas raster
# data(ProtectedAreas)
#
# sdm <- terra::unwrap(CucurbitaRasts)$cordata
# protectedAreas <- terra::unwrap(ProtectedAreas)
#
# #Running checkProtectedAreas
# protectedAreas <- checkProtectedAreas(protectedAreas = protectedAreas,
#                     sdm = sdm)
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra rast values crs same.crs res project
#' @export

checkProtectedAreas <- function(protectedAreas, sdm){
  # check class and convert if needed
  c1 <- class(protectedAreas)
  if(c1[1] !="SpatRaster"){
    protectedAreas <- terra::rast(protectedAreas)
    message(paste("Changed the object type from ", c1,
                  " to the required object terra rast"  ))
  }
  # check to see if this there is only one values in raster
  vals <- unique(terra::values(protectedAreas))[,1]
  if(length(vals) > 2){
    message(paste("Your input raster contain the following values ", vals,
                  " please alter the raster so the that only NA/NaN and 1 are present."  ))
    stop()
  }

  crs <- terra::crs(protectedAreas)
  # crs
  if(!terra::same.crs(crs, terra::crs("epsg:4326"))){
    protectedAreas <- terra::project(x = protectedAreas, y = "epsg:4326" )
    message(paste("Changed the crs from ", crs,
                  " to the required epsg:4326"  ))
  }

  # cell size
  cellSize <- terra::res(protectedAreas)
  modelSize <- terra::res(sdm)
  if(cellSize[1] != modelSize[1] | cellSize[2] != modelSize[2] ){
    protectedAreas <- terra::resample(protectedAreas, sdm, method = "near")
    # this statement is printing twice? not sure why
    message(paste("Changed the cell size of the protected area from ", cellSize,
                  " to the cell size of the sdm ", modelSize  ))
  }
  message(paste("All checks completed"))

  return(protectedAreas)
}

#' @title Quality check on sdm imagery
#' @name checksdm
#' @description
#' Evaluates the class, crs, and values are standardizes to what the following gap analysis functions are required.
#'
#' @param sdm a terra rast object that represented the expected distribution of the species
#'
#' @return sdm : a terra rast object that is in the correct CRS
#'
#' @examples
#' ##Obtaining Raster_list
# data(CucurbitaRasts)
#
# # convert the dataset for function
# sdm <- terra::unwrap(CucurbitaRasts)$cordata
# #Running checksdm
# sdm <- checksdm(sdm = sdm)

#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom terra rast values crs same.crs res project
#' @export
checksdm <- function(sdm){
  # check class and convert if needed
  c1 <- class(sdm)
  if(c1[1]!="SpatRaster"){
    sdm <- terra::rast(sdm)
    message(paste("Changed the object type from ", c1,
                  " to the required object terra rast"  ))
  }
  # check to see if this there is only one values in sdm
  vals <- unique(terra::values(sdm))[,1]
  vals <- vals[!is.na(vals)]
  if(length(vals) > 1 ){
    message(paste("Your input sdm contain the following values ", vals,
                  " please alter the sdm so the that only NA/NaN and 1 are present."  ))
    stop()
  }
  crs <- terra::crs(sdm)
  # crs
  if(!terra::same.crs(crs, terra::crs("epsg:4326"))){
    sdm <- terra::project(x = sdm, y = "epsg:4326" )
    message(paste("Changed the crs from ", crs,
                  " to the required epsg:4326"  ))
  }
  message(paste("All checks completed"))

  return(sdm)
}

#' @title Select relivent ecoregions
#' @name generateEcoSelection
#' @description
#' Utilizes the occurrence data location to select all ecoregions that intersect with thoses points. Helpful as it
#' reduces the overall file size of the ecoregion object.
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#'
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#'
#' @param ecoregions A terra vect object the contains spatial information on all ecoregions of interests
#'
#' @param idColumn A character vector that notes what column within the ecoregions object should be used as a unique ID
#'
#'
#' @return selectedEcos : a terra vect that contains the selected ecoregion features
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' occurrenceData <- CucurbitaData
#' ecoregions <- terra::vect(ecoregions)
#'
#' #Running generateEcoSelection
#' selectedEcos <- generateEcoSelection(taxon = taxon,
#'                     occurrenceData = occurrenceData,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr filter
#' @importFrom terra vect intersect as.data.frame
#' @export

generateEcoSelection <- function(taxon, occurrenceData, ecoregions, idColumn){

  # filter the occurrence data to the species of interest
  d1 <- occurrenceData |>
    dplyr::filter(species == taxon) |>
    terra::vect(geom=c("longitude", "latitude"))

  # determine the eco regions present in the
  inter <- terra::intersect(x = d1, y = ecoregions) |>
    terra::as.data.frame()
  # select ecoregions of interest
  ecoCodes <- unique(inter[,idColumn])
  # index with selection
  ## conver to table for easier indexing
  eco2 <- terra::as.data.frame(ecoregions)
  ## select
  selectedEcos <- ecoregions[eco2[,idColumn] %in% ecoCodes, ]

  # return
  return(selectedEcos)
}

#'
#' @title Final conservation score in situ
#' @name FCSin
#' @description This function calculates the average of the three in situ conservation metrics and
#' assigns a priority category based on the results
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param srsin A dataframe contain the results from the srsin function
#' @param grsin A dataframe contain the results from the grsin function
#' @param ersin A dataframe contain the results from the ersin function
#'
#' @return out_df : a data frames of values summarizing the results of the function
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' occurrenceData <- CucurbitaData
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#' ecoregions <- terra::vect(ecoregions)
#'
#' # generate insitu conservation summaries
#' srs_insitu <- SRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#' grs_insitu <- GRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#' ers_insitu <- ERSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     protectedAreas = protectedAreas,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#' #Running fcsin
#' FCSin <- FCSin(taxon = taxon,
#'                     srsin = srs_insitu,
#'                     grsin = grs_insitu,
#'                     ersin = ers_insitu
#'                     )
#'
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr tibble
#' @export

FCSin <- function(taxon, srsin, grsin, ersin){
  # define variables
  srs <- srsin$results$`SRS insitu`
  grs <- grsin$results$`GRS insitu`
  ers <- ersin$results$`ERS insitu`


  # calculate the mean across the three measures
  sp_fcs <- mean(c(srs,grs,ers), na.rm=TRUE)

  out_df <- dplyr::tibble(Taxon=taxon,
                       "SRS insitu" = srs,
                       "GRS insitu"= grs,
                       "ERS insitu" = ers,
                       "FCS insitu" = sp_fcs,
                       "FCS insitu score" = NA)

  #assign classes (min)
  if (sp_fcs < 25) {
    score <- "UP"
  } else if (sp_fcs >= 25 & sp_fcs < 50) {
    score <- "HP"
  } else if (sp_fcs >= 50 & sp_fcs < 75) {
    score <- "MP"
  } else {
    score <- "LP"
  }
  out_df$"FCS insitu score" <- score
  return(out_df)
}

#' @title Ecological representativeness score ex situ
#' @name ERSex
#' @description The ERSex process provides an ecological measurement of the proportion of a species
#'  range that can be considered to be conserved in ex situ repositories. The ERSex calculates the
#'  proportion of terrestrial ecoregions (The Nature Conservancy Geospatial Conservation Atlas 2019)
#'  represented within the G buffered areas out of the total number of ecoregions occupied by the distribution model.
#'
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param sdm a terra rast object that represented the expected distribution of the species
#' @param occurrenceData a data frame of values containing columns for the taxon, latitude, longitude, and type
#' @param gBuffer A terra vect which encompases a specific buffer distance around all G points
#' @param ecoregions A terra vect object the contains spatial information on all ecoregions of interests
#' @param idColumn A character vector that notes what column within the ecoregions object should be used as a unique ID
#'
#'
#'
#' @return A list object containing
#' 1. results : a data frames of values summarizing the results of the function
#' 2. ecogaps : a terra vect object showing the ecoregions with no area within the g buffer objects
#' 3. map : a leaflet object showing the spatial results of the function
#'
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' ecoregions <- terra::vect(ecoregions)
#' #Running generateGBuffers
#' gBuffer <- generateGBuffers(taxon = taxon,
#'                     occurrenceData = CucurbitaData,
#'                     bufferDistM = 50000
#'                     )
#' #Running ERSex
#' ers_exsitu <- ERSex(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = CucurbitaData,
#'                    gBuffer = gBuffer,
#'                    ecoregions = ecoregions,
#'                    idColumn = "ECO_NAME"
#'                    )
#'
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr filter mutate tibble summarise case_when
#' @importFrom terra vect crop aggregate zonal rasterize as.data.frame
#' @importFrom leaflet addTiles addPolygons addLegend addRasterImage addCircleMarkers
#' @export

ERSex <- function(taxon, sdm, occurrenceData, gBuffer, ecoregions, idColumn) {
  # filter the occurrence data to the species of interest
  d1 <- occurrenceData |>
    dplyr::filter(occurrenceData$species == taxon) |>
    terra::vect(
      geom = c("longitude", "latitude"),
      crs = "+proj=longlat +datum=WGS84"
    )
  # add color
  d1$color <- ifelse(d1$type == "H", yes = "#1184d4", no = "#6300f0")
  # set id column for easier indexing
  ecoregions$id_column <- as.data.frame(ecoregions)[[idColumn]]
  # aggregates spatial features
  ecoregions <- terra::aggregate(x = ecoregions, by = "id_column")

  # determine the eco regions present in the
  ## crop ecos
  ecoregions <- terra::crop(ecoregions, sdm)
  ecoregions$sdmSum <- terra::zonal(
    x = sdm,
    z = ecoregions,
    fun = "sum",
    na.rm = TRUE
  )
  # subset ecoregions to feautres with greated then 0
  ecoSelect <- ecoregions[ecoregions$sdmSum > 0, ]
  # # index with selection
  # ## conver to table for easier indexing
  eco2 <- terra::as.data.frame(ecoSelect) |>
    dplyr::select(ecoID = id_column, count = sdmSum)

  # condition for no G points
  if (is.character(gBuffer$data)) {
    ers <- 0
    gEco <- NA
    gEcoCounts <- 0
    totalEcosCount <- nrow(ecoSelect)
    missingEcos <- eco2$ecoID
  } else {
    # rasterize the buffer object
    b1 <- terra::rasterize(x = gBuffer$data, y = sdm) |> terra::mask(sdm)
    # determine ecoregions in ga50 area
    eco2$bufferEcos <- terra::zonal(
      x = b1,
      z = ecoSelect,
      fun = "sum",
      na.rm = TRUE
    ) |>
      unlist()
    # group by data to get single value per ecoregion
    ecoGrouped <- eco2 |>
      dplyr::mutate(
        bufferEcos = dplyr::case_when(
          is.na(bufferEcos) ~ 0,
          is.nan(bufferEcos) ~ 0,
          TRUE ~ bufferEcos
        )
      ) |>
      dplyr::group_by(ecoID) |>
      dplyr::summarise(
        inDistribution = sum(count, na.rm = TRUE),
        inGBuffer = sum(bufferEcos, na.rm = TRUE)
      )
    # total eco
    totalEcosCount <- nrow(ecoGrouped)
    # ecoregions with coverage
    gEcoIds <- ecoGrouped[ecoGrouped$inGBuffer > 0, "ecoID"] |> pull()
    gEcoCounts <- length(gEcoIds)
    # select map elements
    missingEcos <- ecoSelect[!ecoSelect$id_column %in% gEcoIds, ]
    # ERs calculation
    ers <- min(c(100, (gEcoCounts / totalEcosCount) * 100))
  }

  # generate filter
    out_df = dplyr::tibble(
    Taxon = taxon,
    `Ecoregions with records` = totalEcosCount,
    `Ecoregions within G buffer` = gEcoCounts,
    `ERS exsitu` = ers
  )

  # leaflet map of results
  map_title <- "<h3 style='text-align:center; background-color:rgba(255,255,255,0.7); padding:2px;'>Ecoregions outside of the G Buffer areas</h3>"
  # base map element
  map <- leaflet() |>
    leaflet::addTiles()

  if (nrow(ecoSelect) > 0) {
    map <- map |>
      leaflet::addPolygons(
        data = ecoSelect,
        color = "#444444",
        weight = 1,
        opacity = 1.0,
        fillOpacity = 0.1,
        popup = ~id_column,
        fillColor = NA
      )
  }

  map <- map |>
    leaflet::addLegend(
      position = "topright",
      title = "ERS ex situ",
      colors = c("#47ae24", "#746fae", "#f0a01f", "#44444440"),
      labels = c("Distribution", "G buffer", "Eco gaps", "All Ecos"),
      opacity = 1
    ) |>
    leaflet::addControl(html = map_title, position = "bottomleft")

  if (ers > 0) {
    # add additional map elements
    if (nrow(missingEcos) > 0) {
      map <- map |>
        leaflet::addPolygons(
          data = missingEcos,
          color = "#444444",
          weight = 1,
          opacity = 1.0,
          popup = ~id_column,
          fillOpacity = 0.5,
          fillColor = "#f0a01f"
        )
    }

    map <- map |>
      leaflet::addRasterImage(
        x = sdm,
        colors = "#47ae24"
      ) |>
      leaflet::addRasterImage(
        x = b1,
        colors = "#746fae"
      )
  }

  if (nrow(d1) > 0) {
    map <- map |>
      leaflet::addCircleMarkers(
        data = d1,
        color = ~color,
        radius = 2,
        opacity = 1
      )
  }

  # export results
  output <- list(
    results = out_df,
    ecoGaps = missingEcos,
    map = map
  )

  # generate dataframe
  return(output)
}

#' @title Final Conservation Score measure
#' @name FCSc_mean
#' @description
#' Compiles all tabular data from the individual metrics and generate the final results
#'
#' @param taxon A character object that defines the name of the species as listed in the occurrence dataset
#' @param fcsin A data frame containing summary results from the fcsin function
#' @param fcsex A data frame containing summary results from the fcsex function
#'
#' @return data_comb : a data frame which aggreates final result summaries
#'
#' @examples
#' ##Obtaining occurrences from example
#' data(CucurbitaData)
#' ##Obtaining Raster_list
#' data(CucurbitaRasts)
#' ##Obtaining protected areas raster
#' data(ProtectedAreas)
#' ## ecoregion features
#' data(ecoregions)
#'
#' # convert the dataset for function
#' taxon <- "Cucurbita_cordata"
#' sdm <- terra::unwrap(CucurbitaRasts)$cordata
#' occurrenceData <- CucurbitaData
#' protectedAreas <- terra::unwrap(ProtectedAreas)
#' ecoregions <- terra::vect(ecoregions)
#'
#'
#' # generate exsitu conservation summaries
#' srs_exsitu <- SRSex(taxon = taxon,
#'                     occurrenceData  = CucurbitaData
#'                     )
#'
#' gBuffer <- generateGBuffers(taxon = taxon,
#'                     occurrenceData = occurrenceData,
#'                     bufferDistM = 50000
#'                     )#'
#'
#' grs_exsitu <- GRSex(taxon = taxon,
#'                     sdm = sdm,
#'                     gBuffer = gBuffer
#'                     )
#'
#' ers_exsitu <- ERSex(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     gBuffer = gBuffer,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#' #Running fcsex
#' fcs_exsitu <- FCSex(taxon = taxon,
#'                     srsex = srs_exsitu,
#'                     grsex = grs_exsitu,
#'                     ersex = ers_exsitu
#'                     )
#'
#'
#'
#' # generate insitu conservation summaries
#' srs_insitu <- SRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = CucurbitaData,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#' grs_insitu <- GRSin(taxon = taxon,
#'                     sdm = sdm,
#'                     protectedAreas = protectedAreas
#'                     )
#'
#' ers_insitu <- ERSin(taxon = taxon,
#'                     sdm = sdm,
#'                     occurrenceData = occurrenceData,
#'                     protectedAreas = protectedAreas,
#'                     ecoregions = ecoregions,
#'                     idColumn = "ECO_NAME"
#'                     )
#'
#' #Running fcsin
#' fcs_insitu <- FCSin(taxon = taxon,
#'                     srsin = srs_insitu,
#'                     grsin = grs_insitu,
#'                     ersin = ers_insitu
#'                     )
#'
#' fsc_combine <- FCSc_mean(taxon = taxon,
#'                          fcsin = fcs_insitu,
#'                          fcsex = fcs_exsitu)
#'
#' @references
#' Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
#' Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
#' @importFrom dplyr tibble
#' @export

FCSc_mean <- function(taxon, fcsin, fcsex) {

  #compute FCSc_min and FCSc_max
  data_comb <- dplyr::tibble(Taxon = taxon,
                          "FCS exsitu" = fcsex$`FCS exsitu`,
                          "FCS insitu" = fcsin$`FCS insitu`)


  data_comb$FCSc_min <- min(c(data_comb$`FCS exsitu`,data_comb$`FCS insitu`),na.rm=TRUE)
  data_comb$FCSc_max <- max(c(data_comb$`FCS exsitu`,data_comb$`FCS insitu`),na.rm=TRUE)
  data_comb$FCSc_mean <- mean(c(data_comb$`FCS exsitu`,data_comb$`FCS insitu`),na.rm=TRUE)

  #assign classes (min)
  if (data_comb$FCSc_min < 25) {
    data_comb$FCSc_min_class <- "UP"
  } else if (data_comb$FCSc_min >= 25 & data_comb$FCSc_min < 50) {
    data_comb$FCSc_min_class <- "HP"
  } else if (data_comb$FCSc_min >= 50 & data_comb$FCSc_min < 75) {
    data_comb$FCSc_min_class <- "MP"
  } else {
    data_comb$FCSc_min_class <- "LP"
  }

  #assign classes (max)
  if (data_comb$FCSc_max < 25) {
    data_comb$FCSc_max_class <- "UP"
  } else if (data_comb$FCSc_max >= 25 & data_comb$FCSc_max < 50) {
    data_comb$FCSc_max_class <- "HP"
  } else if (data_comb$FCSc_max >= 50 & data_comb$FCSc_max < 75) {
    data_comb$FCSc_max_class <- "MP"
  } else {
    data_comb$FCSc_max_class <- "LP"
  }

  #assign classes (mean)
  if (data_comb$FCSc_mean < 25) {
    data_comb$FCSc_mean_class <- "UP"
  } else if (data_comb$FCSc_mean >= 25 & data_comb$FCSc_mean < 50) {
    data_comb$FCSc_mean_class <- "HP"
  } else if (data_comb$FCSc_mean >= 50 & data_comb$FCSc_mean < 75) {
    data_comb$FCSc_mean_class <- "MP"
  } else {
    data_comb$FCSc_mean_class <- "LP"
  }

  return(data_comb)
}

---
title: "Gap analysis workflow for multiple species"
output: rmarkdown::html_vignette
knit: (function(input, ...) { out <- "outputs"; if (!dir.exists(out)) dir.create(out, recursive = TRUE); rmarkdown::render(input, output_dir = out) })
vignette: >
  %\VignetteIndexEntry{Gap analysis workflow for multiple species}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
# Check for pandoc and knitr status
has_pandoc <- rmarkdown::pandoc_available()
save_outputs <- !isTRUE(getOption('knitr.in.progress')) && has_pandoc
```


---

This workflow demonstrates how to import your own point data (occurrences) and
SDMs (rasters), check all inputs, and run the full gap analysis for **multiple
species** using the GapAnalysis R package. We provide a general approach using
a for-loop for processing and storing results for multiple species. Example
*Cucurbita* data is provided within the package library.

> **Note:** This vignette recommends [**R version 4.5**](https://cran.r-project.org/) or above. Windows users are also recommended to have [**Rtools 4.5**](https://cran.r-project.org/bin/windows/Rtools/) or above installed.

**For each species, the workflow:**

1. Runs SRSex on all records.
2. Checks inputs (occurrences, rasters, protected areas, ecoregions).
3. Runs gap analysis metrics.
4. Stores results as a table to an output excel file.
5. Creates a conservation priority summary figure as an output png file.
6. Generates interactive leaflet maps for both ex-situ and in-situ conservation as output HTML files.

At script end, the user is prompted to save the workflow by knitting to output HTML.


---

## Download GapAnalysis repository *(ALTERNATIVE)*

Copy the GapAnalysis GitHub repository locally as an alternative method to
source functions and load example data **if the GapAnalysis R package does not
install properly in the step above**. See the
[GitHub resource page](https://docs.github.com/en/get-started/start-your-journey/downloading-files-from-github)
on copying a repository.

> **Easiest method:** Download and extract the
> [GapAnalysis repository](https://github.com/CIAT-DAPA/GapAnalysis/tree/master).
> Select the green [**Code**] button on the main page to download as a ZIP.
>
> You can also clone the repository if you have a GitHub account.

---

## Data preparation

If you downloaded the repository locally, source functions and load data by
removing the hashes below. Your specific path will differ based on your working
directory and the relative path to the downloaded repository.


```{r load-libraries-and-data}
# ── Load libraries ─────────────────────────────────────────────────────────────
library(GapAnalysis)
library(dplyr)
library(tidyr)
library(stringr)
library(terra)
library(leaflet)
library(openxlsx)
library(ggplot2)
library(ggtext)
library(cowplot)
library(htmltools)
library(htmlwidgets)
library(kableExtra)

# ── Set outputs folder ────────────────────────────────────────────────────────
# NOTE: This path must match the path set in the knit: field in the YAML header
# All files will be saved here. Update both locations if changing the path, e.g.:
#outputs_folder <- "C:/Users/you/Desktop/GapAnalysis/outputs"
outputs_folder <- "outputs"
if (!dir.exists(outputs_folder)) dir.create(outputs_folder)

# ── Source functions from GapAnalysis repo (alternative method) ───────────────
#files <- list.files("R", pattern = "\\.[rR]$", full.names = TRUE)
#for (f in files) source(f)

# ── Load data from GapAnalysis repo (alternative method) ─────────────────────
#load("data/CucurbitaData.rda")      # Occurrence data
#load("data/CucurbitaRasts.rda")     # Raster list
#load("data/ProtectedAreas.rda")     # Protected areas raster
#load("data/ecoregions.rda")         # Ecoregions data

# ── Load your own data ────────────────────────────────────────────────────────
# NOTE: To use your own occurrence data and SDM rasters, remove the hashes
# below and update the file paths.
#occData <- read.csv("path/to/your_occurrences.csv") # Local occurrence data
#sdms <- list(                                  # Local rasters read in as list
#   rast("path/to/species1_sdm.tif"),
#   rast("path/to/species2_sdm.tif"))
# NOTE: To download protected areas and ecoregions data locally, run getDatasets(). 
# This requires an active internet connection.
#getDatasets()          # Local download of protected areas and ecoregions data

# ── Load example Cucurbita data (GapAnalysis library) ────────────────────────
# NOTE: To use your own data, hash out the two lines below.
occData <- CucurbitaData                  
sdms    <- terra::unwrap(CucurbitaRasts)

# ── Prepare spatial layers ────────────────────────────────────────────────────
ecos     <- terra::vect(ecoregions)
proAreas <- terra::unwrap(ProtectedAreas)

# Standardize CRS across all spatial layers to prevent mismatch warnings
target_crs <- "+proj=longlat +datum=WGS84"
ecos     <- terra::project(ecos, target_crs)
proAreas <- terra::project(proAreas, target_crs)

# ── Prepare species list and SDM rasters ──────────────────────────────────────
taxa    <- unique(occData$species)
sdmList <- lapply(seq_along(taxa), function(i) {
  r <- sdms[[i]]
  if (terra::crs(r, proj = TRUE) != terra::crs(ecos, proj = TRUE)) {
    r <- terra::project(r, terra::crs(ecos))
  }
  r <- terra::subst(r, 0, NA)
  r[!(is.na(r) | r == 1)] <- NA
  r
})
names(sdmList) <- taxa

# Confirm species names
message("\nConfirm species for gap analysis:\n", paste(taxa, collapse = "\n"))
```

---

## Run gap analysis

Example gap analysis workflow using a named list and for-loop to process and
store results for all species. 

**For details on each metric calculation, review the function documentation
in the [GapAnalysis R folder](https://github.com/CIAT-DAPA/GapAnalysis/tree/master/R).**

```{r run-gap-analysis}
results_list <- list()

for (i in seq_along(taxa)) {
  taxon          <- taxa[i]
  occurrenceData <- occData[occData$species == taxon, ]
  sdm            <- sdms[[i]]

  if (terra::crs(sdm, proj = TRUE) != terra::crs(ecos, proj = TRUE)) {
    sdm <- terra::project(sdm, terra::crs(ecos))
  }
  sdm <- terra::subst(sdm, 0, NA)
  sdm[!(is.na(sdm) | sdm == 1)] <- NA

  # 1. Run SRSex first (on all records)
  srsex <- SRSex(taxon = taxon, occurrenceData = occurrenceData)

  # 2. Check inputs
  occurrences     <- checkOccurrences(csv = occurrenceData, taxon = taxon)
  if (!inherits(sdm, "SpatRaster")) stop("sdm is not a SpatRaster")
  sdm_checked     <- checksdm(sdm)
  proArea_cropped <- terra::crop(proAreas, terra::ext(sdm_checked))
  proArea_checked <- checkProtectedAreas(protectedArea = proArea_cropped,
                                         sdm           = sdm_checked)

  # Skip species if SDM raster has no values
  if (all(is.na(terra::values(sdm_checked)))) {
    message("Skipping ", taxon, ": SDM raster has no values.")
    next
  }

  # 3. Generate G buffers
  gBuffer <- generateGBuffers(
    taxon          = taxon,
    occurrenceData = occurrences$data,
    bufferDistM    = 50000
  )

  # 4. Ex-situ analysis
  grsex <- GRSex(taxon = taxon, sdm = sdm_checked, gBuffer = gBuffer)
  ersex <- ERSex(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    gBuffer        = gBuffer,
    ecoregions     = ecos,
    idColumn       = "ECO_NAME",
    limitByPoints  = FALSE
  )
  fcsex <- FCSex(taxon = taxon, srsex = srsex, grsex = grsex, ersex = ersex)

  # 5. In-situ analysis
  srsin <- SRSin(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    protectedAreas = proArea_checked
  )
  grsin <- GRSin(taxon = taxon, sdm = sdm_checked, protectedAreas = proArea_checked)
  ersin <- ERSin(
    taxon          = taxon,
    sdm            = sdm_checked,
    occurrenceData = occurrences$data,
    protectedAreas = proArea_checked,
    ecoregions     = ecos,
    idColumn       = "ECO_NAME",
    limitByPoints  = FALSE
  )
  fcsin   <- FCSin(taxon = taxon, srsin = srsin, grsin = grsin, ersin = ersin)
  fcsmean <- FCSc_mean(taxon = taxon, fcsin = fcsin, fcsex = fcsex)

  # 6. Store results
  results_list[[taxon]] <- list(
    srsex           = srsex,
    occurrences     = occurrences,
    sdm_checked     = sdm_checked,
    proArea_checked = proArea_checked,
    eco_checked     = ecoregions,
    grsex           = grsex,
    ersex           = ersex,
    fcsex           = fcsex,
    srsin           = srsin,
    grsin           = grsin,
    ersin           = ersin,
    fcsin           = fcsin,
    fcsmean         = fcsmean
  )
}
```

---

## Summary metrics tables

Results are organized as a named list, allowing review of individual species
metrics. The tables below summarize the main metrics for all species.

```{r results-tables}
# ── Ex-situ metrics ───────────────────────────────────────────────────────────
exsitu_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species          = taxon,
    SRS_exsitu       = r$srsex$`SRS exsitu`,
    GRS_exsitu       = r$fcsex$`GRS exsitu`,
    ERS_exsitu       = r$fcsex$`ERS exsitu`,
    FCS_exsitu       = r$fcsex$`FCS exsitu`,
    FCS_exsitu_score = r$fcsex$`FCS exsitu score`
  )
}))

# ── In-situ metrics ───────────────────────────────────────────────────────────
insitu_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species          = taxon,
    SRS_insitu       = r$fcsin$`SRS insitu`,
    GRS_insitu       = r$fcsin$`GRS insitu`,
    ERS_insitu       = r$fcsin$`ERS insitu`,
    FCS_insitu       = r$fcsin$`FCS insitu`,
    FCS_insitu_score = r$fcsin$`FCS insitu score`
  )
}))

# ── Combined metrics ──────────────────────────────────────────────────────────
combined_table <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    Species         = taxon,
    FCSc_min        = r$fcsmean$FCSc_min,
    FCSc_max        = r$fcsmean$FCSc_max,
    FCSc_mean       = r$fcsmean$FCSc_mean,
    FCSc_min_class  = r$fcsmean$FCSc_min_class,
    FCSc_max_class  = r$fcsmean$FCSc_max_class,
    FCSc_mean_class = r$fcsmean$FCSc_mean_class
  )
}))

knitr::kable(exsitu_table,   digits = 2, caption = "<span style='text-align:left; display:block;'>Ex-situ gap analysis metrics</span>",  format = "html") %>% kableExtra::kable_styling()
knitr::kable(insitu_table,   digits = 2, caption = "<span style='text-align:left; display:block;'>In-situ gap analysis metrics</span>",   format = "html") %>% kableExtra::kable_styling()
knitr::kable(combined_table, digits = 2, caption = "<span style='text-align:left; display:block;'>Combined gap analysis metrics</span>",  format = "html") %>% kableExtra::kable_styling()
```

<!-- ## Conservation priority figure -->

```{r gap-analysis-figure, include=FALSE, eval=exists("save_outputs") && save_outputs}
plot_data <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]
  data.frame(
    ID        = taxon,
    SRSex     = as.numeric(r$srsex[["SRS exsitu"]]),
    GRSex     = as.numeric(r$grsex$results[["GRS exsitu"]]),
    ERSex     = as.numeric(r$ersex$results[["ERS exsitu"]]),
    FCSex     = as.numeric(r$fcsex[["FCS exsitu"]]),
    SRSin     = as.numeric(r$srsin$results[["SRS insitu"]]),
    GRSin     = as.numeric(r$grsin$results[["GRS insitu"]]),
    ERSin     = as.numeric(r$ersin$results[["ERS insitu"]]),
    FCSin     = as.numeric(r$fcsin[["FCS insitu"]]),
    FCSc_mean = as.numeric(r$fcsmean[["FCSc_mean"]])
  )
}))

header_font_size     <- 7
legend_label_font_mm <- 5
point_stroke         <- 1.5
other_size           <- 4.0
fcs_size             <- 6.0
fcsc_size            <- 10.5

all_species <- sort(unique(plot_data$ID))
format_id <- function(id) {
  id |>
    str_replace_all(" var\\. ", "</i> var. <i>") |>
    str_replace_all(" x ",      "</i> x <i>") |>
    (\(x) paste0("<i>", x, "</i>"))()
}
formatted_species_levels <- format_id(all_species)

plot_data <- plot_data |>
  mutate(
    ID_italic  = format_id(.data$ID),
    ID_ordered = factor(.data$ID_italic, levels = rev(formatted_species_levels))
  )

long_df <- plot_data |>
  pivot_longer(
    cols      = c(SRSex, GRSex, ERSex, FCSex, SRSin, GRSin, ERSin, FCSin,
                  FCSc_mean),
    names_to  = "metric_raw",
    values_to = "x"
  ) |>
  filter(!is.na(.data$x)) |>
  mutate(
    metric_base = case_when(
      str_detect(.data$metric_raw, "FCSc") ~ "FCSc_mean",
      TRUE ~ str_remove(.data$metric_raw, "(ex|in)$")
    ),
    origin_type = case_when(
      str_detect(.data$metric_raw, "in$")  ~ "in situ",
      str_detect(.data$metric_raw, "ex$")  ~ "ex situ",
      TRUE                                 ~ "combined"
    ),
    point_size_dynamic = case_when(
      .data$metric_base == "FCSc_mean" ~ fcsc_size,
      .data$metric_base == "FCS"       ~ fcs_size,
      TRUE                             ~ other_size
    ),
    plot_order = factor(.data$metric_base,
                        levels = c("FCSc_mean", "FCS", "SRS", "GRS", "ERS"))
  ) |>
  arrange(.data$plot_order)

color_map    <- c("FCSc_mean" = "#FF0000", "FCS" = "#000000",
                  "SRS"       = "#0000FF", "GRS" = "#800080", "ERS" = "#228B22")
shape_map    <- c("combined" = 18, "ex situ" = 16, "in situ" = 17)
panel_colors <- c("#ffb4b3", "#ffd380", "#ffff80", "#a8d2a8")

build_panel <- function(data, id_levels) {
  num_levels <- length(id_levels)
  header_y   <- num_levels + 1.25
  ymax       <- num_levels + 2.0
  ylim_top   <- num_levels + 2.2

  header_data <- tibble(
    x     = c(12.5, 37.5, 62.5, 87.5),
    label = c("<b>Urgent<br>Priority</b>", "<b>High<br>Priority</b>",
              "<b>Medium<br>Priority</b>", "<b>Low<br>Priority</b>")
  )

  ggplot(data, aes(x = .data$x, y = .data$ID_ordered)) +
    annotate("rect",
             xmin  = c(0, 25, 50, 75), xmax = c(25, 50, 75, 100),
             ymin  = 0.5, ymax = ymax,
             fill  = panel_colors, alpha = 0.9) +
    geom_segment(data = data.frame(y = seq_len(num_levels)),
                 aes(x = 0, xend = 100, y = y, yend = y),
                 color = "grey75", linewidth = 0.5, inherit.aes = FALSE) +
    geom_segment(data = data.frame(x = c(0, 25, 50, 75, 100)),
                 aes(x = x, xend = x, y = 0.5, yend = ymax),
                 color = "black", linewidth = 1.5, inherit.aes = FALSE) +
    geom_hline(yintercept = c(0.5, num_levels + 0.5, num_levels + 2.0),
               color = "black", linewidth = 1.5) +
    geom_jitter(aes(color = .data$metric_base, shape = .data$origin_type,
                    size  = .data$point_size_dynamic),
                stroke = point_stroke, width = 0.7, height = 0) +
    geom_richtext(data = header_data,
                  aes(x = .data$x, y = header_y, label = .data$label),
                  size = header_font_size, color = "black",
                  fill = NA, label.color = NA,
                  lineheight = 0.9, hjust = 0.5, inherit.aes = FALSE) +
    scale_color_manual(values = color_map, guide = "none") +
    scale_shape_manual(values = shape_map, guide = "none") +
    scale_size_identity() +
    scale_x_continuous(breaks = c(0, 25, 50, 75, 100), expand = c(0, 0)) +
    scale_y_discrete(drop = FALSE, limits = rev(id_levels)) +
    coord_cartesian(xlim = c(0, 100), ylim = c(1, ylim_top), clip = "off") +
    theme_minimal_grid() +
    theme(
      panel.grid   = element_blank(),
      axis.title   = element_blank(),
      axis.ticks.y = element_blank(),
      axis.ticks.x = element_blank(),
      axis.text.x  = element_text(size = 16),
      axis.text.y  = element_markdown(size = 18, hjust = 1,
                                      margin = margin(r = 15)),
      plot.margin  = margin(t = 30, r = 20, b = 20, l = 40)
    )
}

main_plot <- build_panel(long_df, formatted_species_levels)

legend_data <- tribble(
  ~label,                           ~x,    ~y,   ~shape,    ~color,     ~symbol_size,         ~label_size,          ~vjust,
  "<b>Conservation<br>Status</b>",  0.5,   1.8,  NA_real_,  "black",    NA_real_,             legend_label_font_mm, 1,
  "FCSc-mean",                      3.0,   1.7,  18,        "#FF0000",  fcsc_size,            legend_label_font_mm, 0.5,
  "FCS ex situ",                    4.8,   1.7,  16,        "black",    fcs_size,             legend_label_font_mm, 0.5,
  "SRS ex situ",                    6.6,   1.7,  16,        "#0000FF",  other_size,           legend_label_font_mm, 0.5,
  "GRS ex situ",                    8.4,   1.7,  16,        "#800080",  other_size,           legend_label_font_mm, 0.5,
  "ERS ex situ",                    10.2,  1.7,  16,        "#228B22",  other_size,           legend_label_font_mm, 0.5,
  "FCS in situ",                    4.8,   1.3,  17,        "black",    fcs_size,             legend_label_font_mm, 0.5,
  "SRS in situ",                    6.6,   1.3,  17,        "#0000FF",  other_size,           legend_label_font_mm, 0.5,
  "GRS in situ",                    8.4,   1.3,  17,        "#800080",  other_size,           legend_label_font_mm, 0.5,
  "ERS in situ",                    10.2,  1.3,  17,        "#228B22",  other_size,           legend_label_font_mm, 0.5
)

legend_plot <- ggplot(legend_data, aes(x = .data$x, y = .data$y)) +
  geom_point(aes(shape = .data$shape, color = .data$color,
                 size  = .data$symbol_size), stroke = point_stroke) +
  geom_richtext(aes(label = .data$label, x = .data$x + 0.15,
                    size  = .data$label_size, vjust = .data$vjust),
                hjust = 0, fill = NA, label.color = NA) +
  scale_color_identity() +
  scale_shape_identity() +
  scale_size_identity() +
  theme_void() +
  coord_cartesian(xlim = c(0, 11.5), ylim = c(0.5, 2.2))

n_taxa      <- length(formatted_species_levels)
plot_height <- max(10, n_taxa * 0.5)
final_plot  <- plot_grid(main_plot, legend_plot,
                         ncol = 1, rel_heights = c(1, 0.1))

save_plot(
  file.path(outputs_folder, "gapAnalysis_conservationPriority_figure.png"),
  final_plot,
  base_width  = 12,
  base_height = plot_height,
  dpi         = 300,
  bg          = "white")

message("Saved: ",
        file.path(outputs_folder, "gapAnalysis_conservationPriority_figure.png"))
```

<!-- ## Save metrics to Excel -->

```{r save-metrics-excel, include=FALSE, eval=exists("save_outputs") && save_outputs}
# ── Build table (1 row per species, all metrics as columns) ───────
all_metrics <- do.call(rbind, lapply(names(results_list), function(taxon) {
  r <- results_list[[taxon]]

  clean <- function(x) {
    df <- as.data.frame(t(unlist(x)))
    df <- df[, !grepl("(?i)^(taxon|species)$", names(df)), drop = FALSE]
    return(df)
  }

  row <- cbind(
    data.frame(Taxon = taxon),
    clean(r$srsex),
    clean(r$grsex$results),
    clean(r$ersex$results),
    clean(r$fcsex),
    clean(r$srsin$results),
    clean(r$grsin$results),
    clean(r$ersin$results),
    clean(r$fcsin),
    clean(r$fcsmean)
  )
  row <- row[, !duplicated(names(row)), drop = FALSE]
  return(row)
}))

# ── Round numeric columns ─────────────────────────────────────────────────────
for (col in names(all_metrics)) {
  num <- suppressWarnings(as.numeric(as.character(all_metrics[[col]])))
  if (all(is.na(num) == is.na(all_metrics[[col]]))) {
    all_metrics[[col]] <- round(num, 2)
  }
}

# ── Clean column names ────────────────────────────────────────────────────────
names(all_metrics) <- gsub("\\.", " ", names(all_metrics))

insert_pos  <- which(names(all_metrics) == "Area in protected areas km2")
all_metrics <- cbind(
  all_metrics[, seq_len(insert_pos), drop = FALSE],
  "Area of model km2" = all_metrics[["Area of model km2"]],
  all_metrics[, (insert_pos + 1):ncol(all_metrics), drop = FALSE]
)

names(all_metrics) <- dplyr::recode(names(all_metrics),
  "Total records in SDM"            = "Total records in model",
  "FCSc_min"                        = "FCS minimum",
  "FCSc_max"                        = "FCS maximum",
  "FCSc_mean"                       = "FCSc mean",
  "FCSc_min_class"                  = "FCS minimum priority category",
  "FCSc_max_class"                  = "FCS maximum priority category",
  "FCSc_mean_class"                 = "FCS mean priority category",
  "Ecoregions with protected areas" = "Ecoregions within protected areas"
)

# ── Reorder columns ───────────────────────────────────────────────────────────
col_order <- c(
  "Taxon", "Total records", "Total with coordinates",
  "Total G records", "G records with coordinates",
  "Total H records", "H records with coordinates",
  "SRS exsitu", "G buffer areas in model km2", "Area of model km2",
  "GRS exsitu", "Ecoregions within G buffer", "Ecoregions with records",
  "ERS exsitu", "FCS exsitu", "FCS exsitu score",
  "Total Observations", "Records in Protected areas", "Total records in model",
  "SRS insitu", "Area in protected areas km2", "Area of model km2",
  "GRS insitu", "Ecoregions within protected areas", "Ecoregions within model",
  "ERS insitu", "FCS insitu", "FCS insitu score",
  "FCS minimum", "FCS maximum", "FCSc mean",
  "FCS minimum priority category", "FCS maximum priority category",
  "FCS mean priority category"
)

names(all_metrics) <- make.unique(names(all_metrics), sep = "__TEMP__")
col_order_unique   <- make.unique(col_order,           sep = "__TEMP__")
all_metrics        <- all_metrics[, col_order_unique, drop = FALSE]
names(all_metrics) <- gsub("__TEMP__\\d+$", "", names(all_metrics))

# ── Write to Excel ────────────────────────────────────────────────────────────
wb <- createWorkbook()
addWorksheet(wb, "gapAnalysis_results")
writeData(wb, "gapAnalysis_results", all_metrics, startRow = 1, startCol = 1)
addStyle(wb, "gapAnalysis_results",
  style      = createStyle(wrapText = TRUE, textDecoration = "bold"),
  rows       = 1,
  cols       = seq_len(ncol(all_metrics)),
  gridExpand = TRUE
)
setRowHeights(wb, "gapAnalysis_results", rows = 1, heights = 60)
max_width <- max(nchar(as.character(all_metrics$Taxon))) + 2
setColWidths(wb, "gapAnalysis_results", cols = 1, widths = max_width)
saveWorkbook(wb,
  file      = file.path(outputs_folder, "gapAnalysis_results.xlsx"),
  overwrite = TRUE
)
message("Saved: ", file.path(outputs_folder, "gapAnalysis_results.xlsx"))
```

<!-- ## Interactive maps -->

```{r map-helper, include=FALSE}
clean_map <- function(map, occurrenceData = NULL, taxon = NULL) {

  legend_call <- Filter(
    function(c) !is.null(c$method) && c$method == "addLegend",
    map$x$calls
  )
  labels    <- if (length(legend_call) > 0) legend_call[[1]]$args[[1]][["labels"]] else NULL
  is_insitu <- !is.null(labels) && any(grepl("Protected", labels))

  if (is_insitu) {
    layer_groups <- c("All ecos", "Eco gaps", "Distribution", "Protected areas")
  } else {
    layer_groups <- c("All ecos", "Eco gaps", "Distribution", "G buffer")
  }

  poly_idx <- 0
  map$x$calls <- lapply(map$x$calls, function(call) {
    if (!is.null(call$method) && call$method == "addPolygons") {
      poly_idx <<- poly_idx + 1
      if (poly_idx <= 2) call$args[[3]] <- layer_groups[poly_idx]
    }
    call
  })

  raster_idx <- 0
  map$x$calls <- lapply(map$x$calls, function(call) {
    if (!is.null(call$method) && call$method == "addRasterImage") {
      raster_idx <<- raster_idx + 1
      call$args[[4]] <- layer_groups[2 + raster_idx]
    }
    call
  })

  map$x$calls <- Filter(function(call) {
    !(!is.null(call$method) && call$method == "addCircleMarkers")
  }, map$x$calls)

  map$x$calls <- Filter(function(call) {
    !(!is.null(call$method) && call$method == "addLegend")
  }, map$x$calls)

  map$x$calls <- Filter(function(call) {
    if (!is.null(call$method) && call$method == "addControl") {
      html <- call$args[[1]]
      if (is.character(html) && grepl("Ecoregions", html, ignore.case = TRUE)) {
        return(FALSE)
      }
    }
    TRUE
  }, map$x$calls)

  if (!is.null(occurrenceData) && !is.null(taxon)) {
    occ   <- occurrenceData[occurrenceData$species == taxon, ]
    h_pts <- occ[occ$type == "H", ]
    g_pts <- occ[occ$type == "G", ]

    if (nrow(h_pts) > 0) {
      map <- map |>
        leaflet::addCircleMarkers(
          lng     = h_pts$longitude,
          lat     = h_pts$latitude,
          radius  = 2,
          color   = "#1184d4",
          opacity = 1,
          group   = "H points (herbarium)"
        )
    }
    if (nrow(g_pts) > 0) {
      map <- map |>
        leaflet::addCircleMarkers(
          lng     = g_pts$longitude,
          lat     = g_pts$latitude,
          radius  = 2,
          color   = "#6300f0",
          opacity = 1,
          group   = "G points (genebank)"
        )
    }
  }

  all_groups <- c(layer_groups, "H points (herbarium)", "G points (genebank)")

  map <- map |>
    leaflet::addLayersControl(
      overlayGroups = all_groups,
      options       = layersControlOptions(collapsed = FALSE)
    )

  map <- htmlwidgets::onRender(map, "
    function(el, x) {
      setTimeout(function() {
        el.querySelectorAll('.leaflet-control-layers-overlays').forEach(function(s) {
          var prev = s.previousElementSibling;
          if (prev) prev.remove();
        });
        el.querySelectorAll('.leaflet-bottom.leaflet-left .leaflet-control')
          .forEach(function(c) { c.remove(); });
      }, 300);
    }
  ")

  map
}
```

<!-- ## Render maps -->

```{r render-maps, include=FALSE}
for (taxon in names(results_list)) {
  cat("###", taxon, "\n\n")

  print(htmltools::tagList(
    clean_map(
      map            = results_list[[taxon]]$ersex$map,
      occurrenceData = occData,
      taxon          = taxon
    )
  ))
  cat("\n\n***\n\n")

  print(htmltools::tagList(
    clean_map(
      map            = results_list[[taxon]]$ersin$map,
      occurrenceData = occData,
      taxon          = taxon
    )
  ))
  cat("\n\n***\n\n")
}
```

<!-- ## Save maps as HTML files -->

```{r save-maps-as-html, include=FALSE, eval=exists("save_outputs") && save_outputs}
outputs_abs <- normalizePath(outputs_folder)
tmp_lib     <- file.path(outputs_abs, "lib_tmp")

for (taxon in names(results_list)) {
  clean_name <- tolower(gsub(" ", "_", taxon))

  # Ex-situ map
  ersex_map <- clean_map(
    map            = results_list[[taxon]]$ersex$map,
    occurrenceData = occData,
    taxon          = taxon
  )
  if (!is.null(ersex_map)) {
    file_name <- paste0(clean_name, "_exSitu_map.html")
    htmlwidgets::saveWidget(
      widget        = ersex_map,
      file          = file.path(outputs_abs, file_name),
      selfcontained = TRUE,
      libdir        = tmp_lib
    )
    if (dir.exists(tmp_lib)) unlink(tmp_lib, recursive = TRUE)
    message("Saved: ", file.path(outputs_folder, file_name))
  } else {
    message("No ex-situ map available for: ", taxon)
  }

  # In-situ map
  ersin_map <- clean_map(
    map            = results_list[[taxon]]$ersin$map,
    occurrenceData = occData,
    taxon          = taxon
  )
  if (!is.null(ersin_map)) {
    file_name <- paste0(clean_name, "_inSitu_map.html")
    htmlwidgets::saveWidget(
      widget        = ersin_map,
      file          = file.path(outputs_abs, file_name),
      selfcontained = TRUE,
      libdir        = tmp_lib
    )
    if (dir.exists(tmp_lib)) unlink(tmp_lib, recursive = TRUE)
    message("Saved: ", file.path(outputs_folder, file_name))
  } else {
    message("No in-situ map available for: ", taxon)
  }
}
```

<!-- ## Complete workflow: Knit to save -->

```{r complete-workflow, include=FALSE}
message(
  "\n",
  "──────────────────────────────────────────────────────────────────────\n",
  "                WORKFLOW COMPLETE. WELL DONE!\n",
  "──────────────────────────────────────────────────────────────────────\n",
  "\n",
  " >>> TO SAVE WORKFLOW AS HTML: Click the [Knit] button in RStudio <<<\n",
  "     at the top of the script.\n",
  "\n",
  " The HTML will save to the folder specified in the\n",
  " knit: field of the YAML header.\n",
  "\n",
  " To change the save location, update the path in both:\n",
  "   - the knit: field in the YAML header\n",
  "   - the outputs_folder variable in Data Preparation\n",
  "\n",
  " Review your outputs folder for all saved files.\n",
  "───────────────────────────────────────────────────────────────────────"
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CucurbitaData}
\alias{CucurbitaData}
\title{Cucurbita occurrences dataset}
\format{
A data frame with 1203 rows and 4 variables:
\describe{
  \item{species}{character: Species name}
  \item{latitude}{numeric:Latitude in decimal format}
  \item{longitude}{numeric: Longitude in decimal format}
  \item{type}{character: Source of the record,germplasm (G) or herbarium (H)}
}
}
\source{
\doi{10.7910/DVN/B8YOQL}
}
\usage{
CucurbitaData
}
\description{
This dataset is a subset of the original dataset for:
 C. cordata, C. digitata and C. palmata used in Khoury et al. (2019)
}
\references{
Khoury et al. (2019) Plants, People, Planet 2(3):269-283. \doi{10.1002/ppp3.10085}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ERSex.R
\name{ERSex}
\alias{ERSex}
\title{Ecological representativeness score ex situ}
\usage{
ERSex(taxon, sdm, occurrenceData, gBuffer, ecoregions, idColumn)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{sdm}{a terra rast object that represented the expected distribution of the species}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}

\item{gBuffer}{A terra vect which encompases a specific buffer distance around all G points}

\item{ecoregions}{A terra vect object the contains spatial information on all ecoregions of interests}

\item{idColumn}{A character vector that notes what column within the ecoregions object should be used as a unique ID}
}
\value{
A list object containing
1. results : a data frames of values summarizing the results of the function
2. ecogaps : a terra vect object showing the ecoregions with no area within the g buffer objects
3. map : a leaflet object showing the spatial results of the function
}
\description{
The ERSex process provides an ecological measurement of the proportion of a species
 range that can be considered to be conserved in ex situ repositories. The ERSex calculates the
 proportion of terrestrial ecoregions (The Nature Conservancy Geospatial Conservation Atlas 2019)
 represented within the G buffered areas out of the total number of ecoregions occupied by the distribution model.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
ecoregions <- terra::vect(ecoregions)
#Running generateGBuffers
gBuffer <- generateGBuffers(taxon = taxon,
                    occurrenceData = CucurbitaData,
                    bufferDistM = 50000
                    )
#Running ERSex
ers_exsitu <- ERSex(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = CucurbitaData,
                   gBuffer = gBuffer,
                   ecoregions = ecoregions,
                   idColumn = "ECO_NAME"
                   )


}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ERSin.R
\name{ERSin}
\alias{ERSin}
\title{Ecological representativeness score in situ}
\usage{
ERSin(taxon, sdm, occurrenceData, protectedAreas, ecoregions, idColumn)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{sdm}{a terra rast object that represented the expected distribution of the species}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}

\item{protectedAreas}{A terra rast object the contian spatial location of protected areas.}

\item{ecoregions}{A terra vect object the contains spatial information on all ecoregions of interests}

\item{idColumn}{A character vector that notes what column within the ecoregions object should be used as a unique ID}
}
\value{
A list object containing
1. results : a data frames of values summarizing the results of the function
2. missingEcos : a terra vect object showing all the ecoregions within the distribution with no protected areas present
3. map : a leaflet object showing the spatial results of the function
}
\description{
The ERSin process provides an ecological measurement of the proportion of a species range
 that can be considered to be conserved in protected areas. The ERSin calculates the proportion of ecoregions
 encompassed within the range of the taxon located inside protected areas to the ecoregions encompassed
 within the total area of the distribution model, considering comprehensive conservation to have been accomplished
 only when every ecoregion potentially inhabited by a species is included within the distribution of the species
 located within a protected area.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
protectedAreas <- terra::unwrap(ProtectedAreas)
ecoregions <- terra::vect(ecoregions)

#Running ERSin
ers_insitu <- ERSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = CucurbitaData,
                    protectedAreas = protectedAreas,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )


}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkEcoregion.R
\name{checkEcoregion}
\alias{checkEcoregion}
\title{Quality check of ecoregion dataset}
\usage{
checkEcoregion(ecoregion, sdm, idColumn)
}
\arguments{
\item{ecoregion}{A terra vect object the contains spatial information on all ecoregions of interests}

\item{sdm}{a terra rast object that represented the expected distribution of the species}

\item{idColumn}{A character vector that notes what column within the ecoregions object should be used as a unique ID}
}
\value{
ecoregions : A terra vect object the contains spatial information on all ecoregions of interests
}
\description{
Checks the class, crs, if the idColumn is a unique ID,
}
\examples{
##Obtaining Raster_list

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateCounts.R
\name{generateCounts}
\alias{generateCounts}
\title{Generate initial counts of the occurrence data}
\usage{
generateCounts(taxon, occurrenceData)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}
}
\value{
countsData : a data frames of values summarizing the results of the function
}
\description{
Performs data cleaning to generate a summary of all input occurrence data. These values area
used in the SRSex function.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
occurrenceData <- CucurbitaData

#Running generateCounts
counts <- generateCounts(taxon = taxon,
                    occurrenceData = occurrenceData
                    )

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateEcoSelection.R
\name{generateEcoSelection}
\alias{generateEcoSelection}
\title{Select relivent ecoregions}
\usage{
generateEcoSelection(taxon, occurrenceData, ecoregions, idColumn)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}

\item{ecoregions}{A terra vect object the contains spatial information on all ecoregions of interests}

\item{idColumn}{A character vector that notes what column within the ecoregions object should be used as a unique ID}
}
\value{
selectedEcos : a terra vect that contains the selected ecoregion features
}
\description{
Utilizes the occurrence data location to select all ecoregions that intersect with thoses points. Helpful as it
reduces the overall file size of the ecoregion object.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
occurrenceData <- CucurbitaData
ecoregions <- terra::vect(ecoregions)

#Running generateEcoSelection
selectedEcos <- generateEcoSelection(taxon = taxon,
                    occurrenceData = occurrenceData,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )


}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRSex.R
\name{GRSex}
\alias{GRSex}
\title{Geographical representativeness score ex situ}
\usage{
GRSex(taxon, sdm, gBuffer)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{sdm}{a terra rast object that represented the expected distribution of the species}

\item{gBuffer}{A terra vect which encompases a specific buffer distance around all G points}
}
\value{
A list object containing
1. results : a data frames of values summarizing the results of the function
2. gGaps : a terra vect object showing buffered area about g points
3. map : a leaflet object showing the spatial results of the function
}
\description{
The GRSex process provides a geographic measurement of the proportion of a species’ range
 that can be considered to be conserved in ex situ repositories. The GRSex uses buffers (default 50 km radius)
 created around each G coordinate point to estimate geographic areas already well collected within the distribution
 models of each taxon, and then calculates the proportion of the distribution model covered by these buffers.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
occurrenceData <- CucurbitaData

# generate the g buffer object
gBuffer <- generateGBuffers(taxon = taxon,
                            occurrenceData = occurrenceData,
                            bufferDistM = 50000)

#Running GRSex
grs_exsitu <- GRSex(taxon = taxon,
                    sdm = sdm,
                    gBuffer = gBuffer
                    )



}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SRSex.R
\name{SRSex}
\alias{SRSex}
\title{Sampling representativeness score ex situ}
\usage{
SRSex(taxon, occurrenceData)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}
}
\value{
out_df :  a data frames of values summarizing the results of the function
}
\description{
The SRSex process provides a general indication of the completeness of ex situ conservation collections,
 calculating the ratio of germplasm accessions (G) available in ex situ repositories to reference (H) records for each taxon,
 making use of all compiled records, regardless of whether they include coordinates, with an ideal (i.e., comprehensive) conservation
 ratio of 1:1. In this and in the subsequent measurements, if no G or H records exist, taxa are automatically considered
 to be of high priority for further conservation action and assigned a value of 0. If there are more G than H records,
 SRSex is set to 100.
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)

# convert the dataset for function
taxon <- "Cucurbita_cordata"

#Running SRSex
srs_exsitu <- SRSex(taxon = taxon,
                    occurrenceData = CucurbitaData
                    )

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkOccurrences.R
\name{checkOccurrences}
\alias{checkOccurrences}
\title{Quality check on occurrences data}
\usage{
checkOccurrences(csv, taxon, removeDuplicated = FALSE)
}
\arguments{
\item{csv}{A dataframe holding the occurrence data}

\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{removeDuplicated}{: Binary parameter. TRUE == duplication values are remove. Set to FALSE as default}
}
\value{
A list object containing
1. data : a data frames of values of occurrence data in the required format
2. map : a leaflet object showing the spatial results of the function
}
\description{
Checks the column names, column data types, valid lat lon, and can optionally remove any duplicated lat lon records per species.
The cleaned and formated dataframe is returned as well as a map object show a quick reference of the points in space.
}
\examples{
# example code

##Obtaining occurrences from example


}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ecoregions}
\alias{ecoregions}
\title{Ecoregions vector}
\format{
SpatVector
}
\source{
\doi{10.7910/DVN/B8YOQL}
}
\usage{
ecoregions
}
\description{
This dataset is a subset of the Terrestrial Ecoregions of the World
 shapefile was made available by the world wildlife foundation
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FCSin.R
\name{FCSin}
\alias{FCSin}
\title{Final conservation score in situ}
\usage{
FCSin(taxon, srsin, grsin, ersin)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{srsin}{A dataframe contain the results from the srsin function}

\item{grsin}{A dataframe contain the results from the grsin function}

\item{ersin}{A dataframe contain the results from the ersin function}
}
\value{
out_df : a data frames of values summarizing the results of the function
}
\description{
This function calculates the average of the three in situ conservation metrics and
assigns a priority category based on the results
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
occurrenceData <- CucurbitaData
protectedAreas <- terra::unwrap(ProtectedAreas)
ecoregions <- terra::vect(ecoregions)

# generate insitu conservation summaries
srs_insitu <- SRSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    protectedAreas = protectedAreas
                    )

grs_insitu <- GRSin(taxon = taxon,
                    sdm = sdm,
                    protectedAreas = protectedAreas
                    )

ers_insitu <- ERSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    protectedAreas = protectedAreas,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )

#Running fcsin
FCSin <- FCSin(taxon = taxon,
                    srsin = srs_insitu,
                    grsin = grs_insitu,
                    ersin = ers_insitu
                    )



}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDatasets.R
\name{getDatasets}
\alias{getDatasets}
\title{Download datasets from the harvard dataverse repo}
\usage{
getDatasets()
}
\value{
A message confirming the datasets were downloaded, along with saving the files to the package's data directory.
}
\description{
Ecoregions and protected area data base are stored on a harvard dataverse repository. This functions check to see if
those datasets have been download and will download them if not present.
}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkProtectedAreas.R
\name{checkProtectedAreas}
\alias{checkProtectedAreas}
\title{Quality check on protected areas dataset}
\usage{
checkProtectedAreas(protectedAreas, sdm)
}
\arguments{
\item{protectedAreas}{A terra rast object the contian spatial location of protected areas.}

\item{sdm}{a terra rast object that represented the expected distribution of the species}
}
\value{
protectedAreas : a terra rast object representing protected areas
}
\description{
Checks the class, values, crs, and cell size of the protected areas raster to ensure
these elements match those required by gap analysis functions.
}
\examples{

##Obtaining Raster_list

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRSin.R
\name{GRSin}
\alias{GRSin}
\title{Geographical representativeness score in situ}
\usage{
GRSin(taxon, sdm, protectedAreas)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{sdm}{a terra rast object that represented the expected distribution of the species}

\item{protectedAreas}{A terra rast object the contian spatial location of protected areas.}
}
\value{
A list object containing
1. results : a data frames of values summarizing the results of the function
2. protectAreaMask : a terra rast object showing all the protected areas within the distribution
3. map : a leaflet object showing the spatial results of the function
}
\description{
The GRSin process provides a geographic measurement of the proportion of a species’ range that can be considered
to be conserved in protected areas. The GRSin compares the area of the distribution model located within protected areas versus
the total area of the model, considering comprehensive conservation to have been accomplished only when the entire distribution
occurs within protected areas.
}
\examples{
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
protectedAreas <- terra::unwrap(ProtectedAreas)

#Running GRSin
grs_insitu <- GRSin(taxon = taxon,
                    sdm = sdm,
                    protectedAreas = protectedAreas
                    )



}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkRasters.R
\name{checksdm}
\alias{checksdm}
\title{Quality check on sdm imagery}
\usage{
checksdm(sdm)
}
\arguments{
\item{sdm}{a terra rast object that represented the expected distribution of the species}
}
\value{
sdm : a terra rast object that is in the correct CRS
}
\description{
Evaluates the class, crs, and values are standardizes to what the following gap analysis functions are required.
}
\examples{
##Obtaining Raster_list
}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateGBuffers.R
\name{generateGBuffers}
\alias{generateGBuffers}
\title{Generate buffer of G type occurrences}
\usage{
generateGBuffers(taxon, occurrenceData, bufferDistM)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}

\item{bufferDistM}{Distance in meters. Used to set the size of the buffered objects.}
}
\value{
A list object containing
1. data : a terra vect object showing all the buffered areas around the G type occurrences
2. map : a leaflet object showing the spatial results of the function
}
\description{
Produces a terra vect object representing the area around the G type occurrences
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
occurrenceData <- CucurbitaData

#Running generateGBuffers
gBuffer <- generateGBuffers(taxon = taxon,
                    occurrenceData = occurrenceData,
                    bufferDistM = 50000
                    )



}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FCSc_mean.R
\name{FCSc_mean}
\alias{FCSc_mean}
\title{Final Conservation Score measure}
\usage{
FCSc_mean(taxon, fcsin, fcsex)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{fcsin}{A data frame containing summary results from the fcsin function}

\item{fcsex}{A data frame containing summary results from the fcsex function}
}
\value{
data_comb : a data frame which aggreates final result summaries
}
\description{
Compiles all tabular data from the individual metrics and generate the final results
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
occurrenceData <- CucurbitaData
protectedAreas <- terra::unwrap(ProtectedAreas)
ecoregions <- terra::vect(ecoregions)


# generate exsitu conservation summaries
srs_exsitu <- SRSex(taxon = taxon,
                    occurrenceData  = CucurbitaData
                    )

gBuffer <- generateGBuffers(taxon = taxon,
                    occurrenceData = occurrenceData,
                    bufferDistM = 50000
                    )#'

grs_exsitu <- GRSex(taxon = taxon,
                    sdm = sdm,
                    gBuffer = gBuffer
                    )

ers_exsitu <- ERSex(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    gBuffer = gBuffer,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )

#Running fcsex
fcs_exsitu <- FCSex(taxon = taxon,
                    srsex = srs_exsitu,
                    grsex = grs_exsitu,
                    ersex = ers_exsitu
                    )



# generate insitu conservation summaries
srs_insitu <- SRSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = CucurbitaData,
                    protectedAreas = protectedAreas
                    )

grs_insitu <- GRSin(taxon = taxon,
                    sdm = sdm,
                    protectedAreas = protectedAreas
                    )

ers_insitu <- ERSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    protectedAreas = protectedAreas,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )

#Running fcsin
fcs_insitu <- FCSin(taxon = taxon,
                    srsin = srs_insitu,
                    grsin = grs_insitu,
                    ersin = ers_insitu
                    )

fsc_combine <- FCSc_mean(taxon = taxon,
                         fcsin = fcs_insitu,
                         fcsex = fcs_exsitu)

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CucurbitaRasters}
\alias{CucurbitaRasters}
\alias{CucurbitaRasts}
\title{Cucurbita species distribution models dataset}
\format{
terra rast object stored as a PackedSpatRaster
}
\source{
\doi{10.7910/DVN/B8YOQL}
}
\usage{
CucurbitaRasts
}
\description{
This dataset is a subset of species distribution models for:
 C. cordata, C. digitata and C. palmata used in Khoury et al., 2020
}
\references{
Khoury et al. (2019) Diversity and Distributions 26(2):209-225. \doi{10.1111/DDI.13008}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SRSin.R
\name{SRSin}
\alias{SRSin}
\title{Sampling representativeness score in situ}
\usage{
SRSin(taxon, sdm, occurrenceData, protectedAreas)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{sdm}{a terra rast object that}

\item{occurrenceData}{a data frame of values containing columns for the taxon, latitude, longitude, and type}

\item{protectedAreas}{A terra rast object the contian spatial location of protected areas.}
}
\value{
A list object containing
1. results : a data frames of values summarizing the results of the function
2. points : a terra vect object showing all the points present within protected areas
3. map : a leaflet object showing the spatial results of the function
}
\description{
The SRSin process calculates the proportion of all occurrences of a taxon falling within
the distribution model that also fall within a protected area
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
occurrenceData <- CucurbitaData
protectedAreas <- terra::unwrap(ProtectedAreas)
#Running SRSin
srs_insitu <- SRSin(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    protectedAreas = protectedAreas
                    )

}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FCSex.R
\name{FCSex}
\alias{FCSex}
\title{Final conservation score ex situ}
\usage{
FCSex(taxon, srsex, grsex, ersex)
}
\arguments{
\item{taxon}{A character object that defines the name of the species as listed in the occurrence dataset}

\item{srsex}{A dataframe contain the results from the srsex function}

\item{grsex}{A dataframe contain the results from the grsex function}

\item{ersex}{A dataframe contain the results from the ersex function}
}
\value{
out_df : a data frames of values summarizing the results of the function
}
\description{
This function calculates the average of the three ex situ conservation metrics
  returning a final conservation score summary table. It also assigns conservation priority categories
}
\examples{
##Obtaining occurrences from example
data(CucurbitaData)
##Obtaining Raster_list
data(CucurbitaRasts)
##Obtaining protected areas raster
data(ProtectedAreas)
## ecoregion features
data(ecoregions)

# convert the dataset for function
taxon <- "Cucurbita_cordata"
sdm <- terra::unwrap(CucurbitaRasts)$cordata
occurrenceData <- CucurbitaData
protectedAreas <- terra::unwrap(ProtectedAreas)
ecoregions <- terra::unwrap(ecoregions)
# generate exsitu conservation summaries
srs_exsitu <- SRSex(taxon = taxon,
                    occurrenceData = CucurbitaData
                    )

gBuffer <- generateGBuffers(taxon = taxon,
                    occurrenceData = occurrenceData,
                    bufferDistM = 50000
                    )#'

grs_exsitu <- GRSex(taxon = taxon,
                    sdm = sdm,
                    gBuffer = gBuffer
                    )

ers_exsitu <- ERSex(taxon = taxon,
                    sdm = sdm,
                    occurrenceData = occurrenceData,
                    gBuffer = gBuffer,
                    ecoregions = ecoregions,
                    idColumn = "ECO_NAME"
                    )

#Running fcsex
fcs_exsitu <- FCSex(taxon = taxon,
                    srsex = srs_exsitu,
                    grsex = grs_exsitu,
                    ersex = ers_exsitu)



}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
Carver et al. (2021) GapAnalysis: an R package to calculate conservation indicators using spatial information
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ProtectedAreas}
\alias{ProtectedAreas}
\title{Protected areas dataset in raster format}
\format{
terra rast object
}
\source{
\url{https://www.protectedplanet.net/en}
}
\usage{
ProtectedAreas
}
\description{
This dataset is a raster version of the world protected areas dataset
 used in Khoury et al., (2019)
}
\references{
Khoury et al. (2019) Ecological Indicators 98:420-429. \doi{10.1016/j.ecolind.2018.11.016}
}
\keyword{datasets}
